; ********************************************************
; ** Name:    s88.asm                                   **
; ** Version: V0.15 vom 04/03/2022                      **
version equ 015h
; ** Project:                                           **
; **                                                    **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; ** Prozessor: Pic 16F690 / Microchip                  **
; **            1 cycle = 0.5 us (8 MHz internal Osc)   **
; **                                                    **
; ** IR: RB0 for Input-Signal (COMMAND)                 **
; **                                                    **
; ** Timer: 0 for LED                                   **
; **        1 not used                                  **
; **        2 not used                                  **
; **                                                    **
; ** WDT: Timeout after 18 ms                           **
; **                                                    **
; ********************************************************
; Copyright (C) 2011-2022 Sven Brandt

; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License version 2send_co
; as published by the Free Software Foundation.

; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

#ifndef makefile

;#define dsd2010_grube
;#define gls88
;#define as88
;#define as88_2v6
#define ms88
;#define weich88

;#define test                ; aktiviert diverse Test-Toggler
;#define test_use_DIP_as_OUT ; C5/C7 sind OUTPUT fr Toggle-Funktioen, DIP muss OFF sein!!!
;#define test_s88            ; aktiviert Toggle-Funktionen fr Debug S88-Schnittstelle
;#define test_s88_bit        ; DSD 64 Bit Test
;#define sende_testdaten     ; sendet festes Bitmuster ber S88

;#define test_osccal   ; generiert Impulse an IO_LED_STATUS von 2 us (4 Zyklen) = 250 kHz

#endif ; makefile

#ifdef dsd2010_grube
;#define polarity_ps_inverted             ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_clk_inverted            ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_data_in_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#define polarity_data_out_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#define dsd2010_grube_use_mode_5          ; neuer Mode mit 64 Bis ab V0.04
#endif ; dsd2010_grube

#ifdef gls88
;#define polarity_ps_inverted             ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_clk_inverted            ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_data_in_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#define polarity_data_out_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#endif ; gls88

#ifdef as88
;#define polarity_ps_inverted             ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_clk_inverted            ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_data_in_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#define polarity_data_out_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#endif ; as88

#ifdef as88_2v6
;#define polarity_ps_inverted             ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_clk_inverted            ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_data_in_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#define polarity_data_out_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#define no_led_status                     ; es gibt keine LED_STATUS, sondern einen 4094
#endif ; as88_2v6

#ifdef ms88
;#define polarity_ps_inverted             ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_clk_inverted            ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_data_in_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#define polarity_data_out_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#endif ; ms88

#ifdef weich88
;#define polarity_ps_inverted             ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_clk_inverted            ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
;#define polarity_data_in_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#define polarity_data_out_inverted        ; **** define setzen, wenn am PIC! invertierte Signale liegen ***
#endif ; weich88

 
; ********************************************************
; History:
; ********************************************************
; V0.15 - 2022-03-03 - DSD2010 Korrektur BIT#1 (zu langsame Abfrage 4094-Eingang -> Blinken des Bits #1)
;                    - MS88: DIP2 -> Eingnge invertieren
; V0.14 - 2021-10-31 - as88_2v6 hinzu, hier werden 8 LED via 4094 angesteuert, 
; V0.13 - 2020-12-21 - DSD: fr DSD keine DEBOUNCE Funktion mehr
; V0.12 - 2018-02-22 - AS: Einlese-Lnge (Abschalten der LEDs) von ca. 16 us auf 30 us verlngert, berarbeitete Einlese-Routine
; V0.11 - 2018-02-09 - in ISR wird nicht mehr bei jedem CLK zunchst DATA_OUT auf LOW gesetzt (Zeitgewinn nur 0.5 us) 
; V0.11 - 2018-02-08 - WPUA Pull Ups fr Port A (#0, #1)
; V0.10 - 2018-01-19 - Anpassung MS88 an CS3
; V0.09 - 2017-11-18 - Version AS88 hinzu
; V0.08 - 2016-08-14 - Version MS88 hinzu
; V0.07 - 2015-05-22 - Debounce Funktion hinzu fr DSD (S88 Output Daten mssen 2x identisch sein vor der Ausgabe auf den S88)
; V0.06 - 2015-05-22 - Debounce Funktion fr DSD 1. Hlfte / 2. Hlfte
; V0.05 - 2015-05-20 - berarbeitung ISR (da CS2 60215 nur ca. 14 us LOW Phase im Clock hat)
; V0.04 - 2015-04-19 - DSD: bitorientiert = 48 Bits
; V0.03 - 2014-09-29 - Version fr Weich88-N hinzu
; V0.03 - 2011-10-07 - Optimierung Timing, insbesondere fr ECoS und Tams (S88-CLK sehr lange HIGH)
;                    - bernahme der empf. Bits nur, wenn Anzahl korrekt ist
;                    - Korrektur DIP-Schalter Zuordnung
;                    - LED blinkt langsamer, wenn kein S88-Signal (Reduzierung der Auslastung)
; V0.01 - 2011-04-13 - Neuerstellung


;   --------------------------------------------------------------------------
;   --- Preludium
;   --------------------------------------------------------------------------
 __idlocs h'8000' | version            ; PIC Device ID

 ERRORLEVEL 0,-302                   ; Errorlevel
 #include <p16f690.inc>
 LIST p=16f690                       ; Controller

;   --------------------------------------------------------------------------
;   --- CONFIG
;   --------------------------------------------------------------------------
 __config b'00001111001100'          ; Programmer-Configuration
;           ++---------------- #13-#12 Reserved: Reserved bits. Do Not Use.
;             +--------------- #11 FCMEN: Fail-Safe Clock Monitor Enabled bit
;              |||||||||||         1 = Fail-Safe Clock Monitor is enabled    
;              |||||||||||         0 = Fail-Safe Clock Monitor is disabled   
;              +-------------- #10 IESO: Internal External Switchover bit          
;               ||||||||||         1 = Internal External Switchover mode is enabled
;               ||||||||||         0 = Internal External Switchover mode is disabled
;               ++------------ #9-#8 BOREN<1:0>: Brown-out Reset Selection bits(1)          
;                 ||||||||        >11 = BOR enabled                                       
;                 ||||||||         10 = BOR enabled during operation and disabled in Sleep
;                 ||||||||         01 = BOR controlled by SBOREN bit of the PCON register 
;                 ||||||||         00 = BOR disabled                                      
;                 +----------- #7: CPD: Data Code Protection bit(2)           
;                  |||||||        >1 = Data memory code protection is disabled
;                  |||||||         0 = Data memory code protection is enabled 
;                  +---------- #6: CP: Code Protection bit(2)                    
;                   ||||||        >1 = Program memory code protection is disabled
;                   ||||||         0 = Program memory code protection is enabled 
;                   +--------- #5: MCLRE: MCLR Pin Function Select bit(4)
;                    |||||         1 = MCLR pin function is MCLR         
;                    |||||        >0 = MCLR pin function is digital input, MCLR internally tied to VDD
;                    +-------- #4: PWRTE: Power-up Timer Enable bit
;                     ||||         1 = PWRT disabled               
;                     ||||        >0 = PWRT enabled                
;                     +------- #3: WDTE: Watchdog Timer Enable bit
;                      |||        >1 = WDT enabled                
;                      |||         0 = WDT disabled               
;                      +++---- #2-#0: FOSC<2:0>: Oscillator Selection bits                                                   
;                                  111 = RC oscillator: CLKOUT function on RA4/OSC2/CLKOUT pin, RC on RA5/OSC1/CLKIN      
;                                  110 = RCIO oscillator: I/O function on RA4/OSC2/CLKOUT pin, RC on RA5/OSC1/CLKIN       
;                                  101 = INTOSC oscillator: CLKOUT function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN                                                         
;                                 >100 = INTOSCIO oscillator: I/O function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN                                                         
;                                  011 = EC: I/O function on RA4/OSC2/CLKOUT pin, CLKIN on RA5/OSC1/CLKIN                 
;                                  010 = HS oscillator: High-speed crystal/resonator on RA4/OSC2/CLKOUT and RA5/OSC1/CLKIN
;                                  001 = XT oscillator: Crystal/resonator on RA4/OSC2/CLKOUT and RA5/OSC1/CLKIN           
;                                  000 = LP oscillator: Low-power crystal on RA4/OSC2/CLKOUT and RA5/OSC1/CLKIN           

;   --------------------------------------------------------------------------
;   --- Register & Flags
;   --------------------------------------------------------------------------
;       20h ..  7Fh bank 0
;       A0h ..  EFh bank 1
;      120h .. 16Fh bank 2
;   --------------------------------------------------------------------------

R_PORTB                    equ 20h          ; for saving PORTB
R_PORTC                    equ 21h          ; for saving PORTC

R_FLAG1                    equ 22h          ; various flags
 ;#define       F_FREEZE          R_FLAG1,0  ; nur AS: 0 = if DIP_1 (C7) = LOW = ON  -> freeze / 1 = if DIP_1 (C7) = HIGH = OFF -> no freeze
 #define       F_PS_OK           R_FLAG1,1  ; 1 = PS ist gekommen
R_BUSERKENNUNG             equ 23h          ; Register for debouncing S88 Buserkennung (Counter f. Erkennunf S88 not OK -> LED blinken)
R_TMR0_CNT                 equ 24h          ; Counter for LED flashing (blinkt sonst zu schnell und belastst zu sehr - Tams!)

; Register fr debouncing
RAM_R_DEB                  equ h'25' ; fr indirekte Adressierung
R_DEB1                     equ RAM_R_DEB    ; debouncing counter #1 h'25'
R_DEB2                     equ RAM_R_DEB+1  ; debouncing counter #2 h'26'
R_DEB3                     equ RAM_R_DEB+2  ; debouncing counter #3 h'27'
R_DEB4                     equ RAM_R_DEB+3  ; debouncing counter #4 h'28'
R_DEB5                     equ RAM_R_DEB+4  ; debouncing counter #5 h'29'
R_DEB6                     equ RAM_R_DEB+5  ; debouncing counter #6 h'2a'
R_DEB7                     equ RAM_R_DEB+6  ; debouncing counter #7 h'2b'
R_DEB8                     equ RAM_R_DEB+7  ; debouncing counter #8 h'2c'

; ISR      
W_TEMP                     equ 2eh          ; saving W-Register
STATUS_TEMP                equ 2fh          ; saving Status-Register
R_S88_CLK_TIMEOUT          equ 3ch          ; S88 CLK Timeout: Nach xx us CLOCK High wird trotzdem DATA eingelesen (Problem CS2-> kurze LOW Phase auf dem CLK)

; nur fr DSD2010
R_S88BIT_ANZAHL            equ 30h          ; Anzahl der S88 Bits (abhngig von Mode)
R_MODE                     equ 31h          ; bertragungs-Mode
 #define       F_MODE_01         R_MODE,0   ; S88 bertragungs-Mode 1: DIP1 = OFF, DIP2 = OFF
 #define       F_MODE_02         R_MODE,1   ; S88 bertragungs-Mode 2: DIP1 = OFF, DIP2 = ON
 #define       F_MODE_03         R_MODE,2   ; S88 bertragungs-Mode 3: DIP1 = ON,  DIP2 = OFF
 #define       F_MODE_04         R_MODE,3   ; S88 bertragungs-Mode 4: DIP1 = ON, DIP2 = ON
 #define       F_MODE_05         R_MODE,4   ; S88 bertragungs-Mode 5: DIP1 = ON, DIP2 = ON @ #define dsd2010_grube_use_mode_5
R_TEMP                    equ 32h           ; Temp Register
R_DSD_FLAGS               equ 33h           ; various Flags
 #define       F_RM_CLK_OLD      R_DSD_FLAGS,0  ; Flanken-Erkennung RM_CLK
 #define       F_RM_DATA         R_DSD_FLAGS,1  ; zwischenspeichern des Daten-Eingangs (vin Grube-Main)

; die Eingangs-Daten werden in die Register _IN_ mit den RM_CLK Flanken geschoben. Bei RM_STROBE
; werden diese Daten in die Register _OUT_ bernommen.
; Die _OUT_Register hingegen werden beim PS_S88 in die Register _STREAM_ bernommen
; und damit in den S88 Datenstrom eingeschleift.
R_DATA_IN_CLK             equ 34h       ; history CLK to get positive edge
R_DATA_IN_1               equ 35h       ; bertragungs-Daten (Eingang Optokoppler R_STAT_FLAG_02)
 #define      F_SEC_HALF         R_DATA_IN_1,7 ; 1 = Position in der "2. Hlfte"
R_DATA_IN_2               equ 36h       ; bertragungs-Daten (Eingang Optokoppler R_ISTPOSITION)
R_DATA_IN_3               equ 37h       ; bertragungs-Daten (Eingang Optokoppler tbd)
R_DATA_IN_4               equ 38h       ; bertragungs-Daten (Eingang Optokoppler R_ERRORCODE)
R_DATA_IN_1_OLD           equ 39h       ; bertragungs-Daten (Eingang Optokoppler R_STAT_FLAG_02)
 #define      F_SEC_HALF_OLD     R_DATA_IN_1_OLD,7  ; 1 = Position in der "2. Hlfte"

R_DATA_COUNTER            equ 3ah       ; Counter zhlt empfangene Bits -> verwerfen von Infos mit fehlenden Bits

; Timing
R_TIMECOUNT_1             equ 3dh          ; Loop: Counter 1 fr langsame Ablufe
R_TIMECOUNT_2             equ 3eh          ; Loop: Counter 2 fr langsame Ablufe

; S88 Daten 
RAM_R_S88DATA_OUT         equ h'40'    ; fr indirekte Adressierung
R_S88DATA_OUT_0           equ RAM_R_S88DATA_OUT      ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 1. Byte auf dem Bus
#ifdef dsd2010_grube  ; nur DSD hat mehr als 1 Byte
R_S88DATA_OUT_1           equ RAM_R_S88DATA_OUT+1    ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 2. Byte auf dem Bus
R_S88DATA_OUT_2           equ RAM_R_S88DATA_OUT+2    ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 3. Byte auf dem Bus
R_S88DATA_OUT_3           equ RAM_R_S88DATA_OUT+3    ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 4. Byte auf dem Bus
R_S88DATA_OUT_4           equ RAM_R_S88DATA_OUT+4    ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 5. Byte auf dem Bus
R_S88DATA_OUT_5           equ RAM_R_S88DATA_OUT+5    ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 6. Byte auf dem Bus
R_S88DATA_OUT_6           equ RAM_R_S88DATA_OUT+6    ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 7. Byte auf dem Bus
R_S88DATA_OUT_7           equ RAM_R_S88DATA_OUT+7    ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 8. Byte auf dem Bus
#endif ; dsd2010_grube

RAM_R_S88DATA_PRE_A equ h'48'    ; fr indirekte Adressierung
R_S88DATA_PRE_A_0         equ RAM_R_S88DATA_PRE_A    ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 1. Byte auf dem Bus - Debounce Latch A
#ifdef dsd2010_grube  ; nur DSD hat mehr als 1 Byte
R_S88DATA_PRE_A_1         equ RAM_R_S88DATA_PRE_A+1  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 2. Byte auf dem Bus - Debounce Latch A
R_S88DATA_PRE_A_2         equ RAM_R_S88DATA_PRE_A+2  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 3. Byte auf dem Bus - Debounce Latch A
R_S88DATA_PRE_A_3         equ RAM_R_S88DATA_PRE_A+3  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 4. Byte auf dem Bus - Debounce Latch A
R_S88DATA_PRE_A_4         equ RAM_R_S88DATA_PRE_A+4  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 5. Byte auf dem Bus - Debounce Latch A
R_S88DATA_PRE_A_5         equ RAM_R_S88DATA_PRE_A+5  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 6. Byte auf dem Bus - Debounce Latch A
R_S88DATA_PRE_A_6         equ RAM_R_S88DATA_PRE_A+6  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 7. Byte auf dem Bus - Debounce Latch A
R_S88DATA_PRE_A_7         equ RAM_R_S88DATA_PRE_A+7  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 8. Byte auf dem Bus - Debounce Latch A
#endif ; dsd2010_grube

RAM_R_S88DATA_PRE_B       equ h'50'    ; fr indirekte Adressierung
R_S88DATA_PRE_B_0         equ RAM_R_S88DATA_PRE_B    ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 1. Byte auf dem Bus - Debounce Latch B
#ifdef dsd2010_grube  ; nur DSD hat mehr als 1 Byte
R_S88DATA_PRE_B_1         equ RAM_R_S88DATA_PRE_B+1  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 2. Byte auf dem Bus - Debounce Latch B
R_S88DATA_PRE_B_2         equ RAM_R_S88DATA_PRE_B+2  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 3. Byte auf dem Bus - Debounce Latch B
R_S88DATA_PRE_B_3         equ RAM_R_S88DATA_PRE_B+3  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 4. Byte auf dem Bus - Debounce Latch B
R_S88DATA_PRE_B_4         equ RAM_R_S88DATA_PRE_B+4  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 5. Byte auf dem Bus - Debounce Latch B
R_S88DATA_PRE_B_5         equ RAM_R_S88DATA_PRE_B+5  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 6. Byte auf dem Bus - Debounce Latch B
R_S88DATA_PRE_B_6         equ RAM_R_S88DATA_PRE_B+6  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 7. Byte auf dem Bus - Debounce Latch B
R_S88DATA_PRE_B_7         equ RAM_R_S88DATA_PRE_B+7  ; bertragungs-Daten (Ausgang S88) - Daten dieses Moduls - 8. Byte auf dem Bus - Debounce Latch B
#endif ; dsd2010_grube

R_DATA_TEMP1              equ 58h       ; temp. Register fr RLF Funktion (sonst Problem bei PS whrend der RLF Funktion)
R_DATA_TEMP2              equ 59h       ; temp. Register fr RLF Funktion (sonst Problem bei PS whrend der RLF Funktion)
R_DATA_TEMP3              equ 5ah       ; temp. Register fr RLF Funktion (sonst Problem bei PS whrend der RLF Funktion)

R_4094_VALUES_1          equ  5bh     ; Byte 1 for 4014 contents
R_4094_VALUES_1_OLD      equ  5ch     ; Byte 1 for 4014 contents
R_4094_FLAGS             equ  5dh
  #define F_4094_UPDATE       R_4094_FLAGS,0 ; 4094 Updaten
  #define F_4094_MODE_INPUT   R_4094_FLAGS,1 ; Inputs anzeigen
  #define F_4094_MODE_KNIGHT  R_4094_FLAGS,2 ; Lauflicht generieren
  #define F_4094_KNIGHT_LEFT  R_4094_FLAGS,3 ; Lauflicht Links herum

R_S88DATA_STREAM_0        equ 60h       ; bertragungs-Daten (Ausgang S88) - Datenstrom (Daten dieses Moduls if PS=active, sonst Daten der Folge-Module)
#ifdef dsd2010_grube  ; nur DSD hat mehr als 1 Byte
R_S88DATA_STREAM_1        equ 61h       ; bertragungs-Daten (Ausgang S88) - Datenstrom (Daten dieses Moduls if PS=active, sonst Daten der Folge-Module)
R_S88DATA_STREAM_2        equ 62h       ; bertragungs-Daten (Ausgang S88) - Datenstrom (Daten dieses Moduls if PS=active, sonst Daten der Folge-Module)
R_S88DATA_STREAM_3        equ 63h       ; bertragungs-Daten (Ausgang S88) - Datenstrom (Daten dieses Moduls if PS=active, sonst Daten der Folge-Module)
R_S88DATA_STREAM_4        equ 64h       ; bertragungs-Daten (Ausgang S88) - Datenstrom (Daten dieses Moduls if PS=active, sonst Daten der Folge-Module)
R_S88DATA_STREAM_5        equ 65h       ; bertragungs-Daten (Ausgang S88) - Datenstrom (Daten dieses Moduls if PS=active, sonst Daten der Folge-Module)
R_S88DATA_STREAM_6        equ 66h       ; bertragungs-Daten (Ausgang S88) - Datenstrom (Daten dieses Moduls if PS=active, sonst Daten der Folge-Module)
R_S88DATA_STREAM_7        equ 67h       ; bertragungs-Daten (Ausgang S88) - Datenstrom (Daten dieses Moduls if PS=active, sonst Daten der Folge-Module)
#endif ;dsd2010_grube

RAM_R_PS_COUNT            equ h'70'    ; fr indirekte Adressierung
R_PS_COUNT_1              equ RAM_R_PS_COUNT     ; Counter IN1: Zhlt mit jedem PS Impuls runter, sodass DATA fr mindestens x S88-Perioden ansteht
R_PS_COUNT_2              equ RAM_R_PS_COUNT+1   ; Counter IN2: Zhlt mit jedem PS Impuls runter, sodass DATA fr mindestens x S88-Perioden ansteht
R_PS_COUNT_3              equ RAM_R_PS_COUNT+2   ; Counter IN3: Zhlt mit jedem PS Impuls runter, sodass DATA fr mindestens x S88-Perioden ansteht
R_PS_COUNT_4              equ RAM_R_PS_COUNT+3   ; Counter IN4: Zhlt mit jedem PS Impuls runter, sodass DATA fr mindestens x S88-Perioden ansteht
R_PS_COUNT_5              equ RAM_R_PS_COUNT+4   ; Counter IN5: Zhlt mit jedem PS Impuls runter, sodass DATA fr mindestens x S88-Perioden ansteht
R_PS_COUNT_6              equ RAM_R_PS_COUNT+5   ; Counter IN6: Zhlt mit jedem PS Impuls runter, sodass DATA fr mindestens x S88-Perioden ansteht
R_PS_COUNT_7              equ RAM_R_PS_COUNT+6   ; Counter IN7: Zhlt mit jedem PS Impuls runter, sodass DATA fr mindestens x S88-Perioden ansteht
R_PS_COUNT_8              equ RAM_R_PS_COUNT+7   ; Counter IN8: Zhlt mit jedem PS Impuls runter, sodass DATA fr mindestens x S88-Perioden ansteht

R_FLAGS_FREEZE            equ 78h  ; ermglicht MS88, die Bits fr eine bestimmte Anzahl von Abfragen auf den Bus zu legen
#define  FREEZE_OUT1         R_FLAGS_FREEZE,0 ; 1 = #1 bleibt aktiv fr eine eingestelle Anzahl PS Impulse = S88 Zyklen)
#define  FREEZE_OUT2         R_FLAGS_FREEZE,1 ; 1 = #2 bleibt aktiv fr eine eingestelle Anzahl PS Impulse = S88 Zyklen)
#define  FREEZE_OUT3         R_FLAGS_FREEZE,2 ; 1 = #3 bleibt aktiv fr eine eingestelle Anzahl PS Impulse = S88 Zyklen)
#define  FREEZE_OUT4         R_FLAGS_FREEZE,3 ; 1 = #4 bleibt aktiv fr eine eingestelle Anzahl PS Impulse = S88 Zyklen)
#define  FREEZE_OUT5         R_FLAGS_FREEZE,4 ; 1 = #5 bleibt aktiv fr eine eingestelle Anzahl PS Impulse = S88 Zyklen)
#define  FREEZE_OUT6         R_FLAGS_FREEZE,5 ; 1 = #6 bleibt aktiv fr eine eingestelle Anzahl PS Impulse = S88 Zyklen)
#define  FREEZE_OUT7         R_FLAGS_FREEZE,6 ; 1 = #7 bleibt aktiv fr eine eingestelle Anzahl PS Impulse = S88 Zyklen)
#define  FREEZE_OUT8         R_FLAGS_FREEZE,7 ; 1 = #8 bleibt aktiv fr eine eingestelle Anzahl PS Impulse = S88 Zyklen)

R_INPUTLOOP_FLAG          equ 79h         ; in der Abfrage-Loop: Flag, welcher Eingang behandelt wird  (fr Maskierungen & Flags)
R_INPUTLOOP_CNT           equ 7ah         ; in der Abfrage-Loop: Counter (fr indirekte Adressierung)
R_INPUTS                  equ 7bh         ; in dieses Register werden die abgefragten Eingnge abgelegt

;   --------------------------------------------------------------------------
;   --- Port-Definitionen
;   --------------------------------------------------------------------------
#ifdef dsd2010_grube
  #define IO_P_DATA     PORTA,0        ;Layout Definition, Input (Programming)
  #define IO_P_CLK      PORTA,1        ;Layout Definition, Input (Programming)
  #define IO_S88_CLK    PORTA,2        ;Layout Definition, Input IR (S88) 
  #define IO_VPP        PORTA,3        ;Layout Definition, Input (Input only) (Programming)
  #define IO_S88_PS     PORTA,4        ;Layout Definition, Input (S88)
  #define IO_DATA_IN    PORTA,5        ;Layout Definition, Input (S88)
  
  #define IO_RM_STROBE  PORTB,4        ;Layout Definition, Input (Optokoppler) 
  #define IO_DATA_OUT   PORTB,5        ;Layout Definition, Output (S88)
  #define IO_LED_STATUS PORTB,6        ;Layout Definition, Output (LED)
  #define IO_free1      PORTB,7        ;Layout Definition, Output (unused)
  
  #define IO_RM_RESERVE PORTC,0        ;Layout Definition, Input (optokoppler)
  #define IO_RM_CLK     PORTC,1        ;Layout Definition, Input (Optokoppler)
  #define IO_RM_DATA    PORTC,2        ;Layout Definition, Input (Optokoppler)
  #define IO_test_3     PORTC,3        ;Layout Definition, Output (unused)
  #define IO_test_2     PORTC,4        ;Layout Definition, Output (unused)
  #define IO_test_1     PORTC,5        ;Layout Definition, Output (unused)
  #define IO_DIP2       PORTC,6        ;Layout Definition, Input (DIP2) 
  #define IO_DIP1       PORTC,7        ;Layout Definition, Input (DIP1)
#endif ;dsd2010_grube

#ifdef as88
  #define IO_POWER_LED  PORTA,0        ;Layout Definition, Output (Anzeige-LED)
  #define IO_P_CLK      PORTA,1        ;Layout Definition, Input (Programming)
  #define IO_S88_CLK    PORTA,2        ;Layout Definition, Input IR (S88) 
  #define IO_FREEZE     PORTA,3        ;Layout Definition, Input (Input only) (Programming)
  #define IO_S88_PS     PORTA,4        ;Layout Definition, Input (S88)
  #define IO_DATA_IN    PORTA,5        ;Layout Definition, Input (S88)
  
  #define IO_RM4        R_PORTB,4      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DATA_OUT   PORTB,5        ;Layout Definition, Output (S88)
  #define IO_RM6        R_PORTB,6      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_LED_STATUS PORTB,7        ;Layout Definition, Output (LED)
  
  #define IO_RM1        R_PORTC,0      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM2        R_PORTC,1      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM3        R_PORTC,2      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM8        R_PORTC,3      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM5        R_PORTC,4      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DIP2       PORTC,5        ;Layout Definition, Input (DIP2 -> 1 = Inputs invertiert)
  #define IO_RM7        R_PORTC,6      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DIP1       PORTC,7        ;Layout Definition, Input (DIP1 -> ON = LOW = FREEZE / OFF = HIGH = kein FREEZE
#endif ;as88

#ifdef as88_2v6
  #define IO_STROBE_4094 PORTA,0        ;Layout Definition, Output (LEDs via 4094)
  #define IO_DATA_4094   PORTA,1        ;Layout Definition, Output (LEDs via 4094)
  #define IO_S88_CLK    PORTA,2        ;Layout Definition, Input INT (S88) 
  #define IO_FREEZE     PORTA,3        ;Layout Definition, Input (Input only) (Programming)
  #define IO_S88_PS     PORTA,4        ;Layout Definition, Input (S88)
  #define IO_DATA_IN    PORTA,5        ;Layout Definition, Input (S88)
  ;
  #define IO_RM4        R_PORTB,4      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DATA_OUT   PORTB,5        ;Layout Definition, Output (S88)
  #define IO_RM6        R_PORTB,6      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_CLK_4094   PORTB,7        ;Layout Definition, Output (LEDs via 4094)
  ;
  #define IO_RM1        R_PORTC,0      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM2        R_PORTC,1      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM3        R_PORTC,2      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM8        R_PORTC,3      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM5        R_PORTC,4      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DIP2       PORTC,5        ;Layout Definition, Input (DIP2 -> 1 = Inputs invertiert)
  #define IO_RM7        R_PORTC,6      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DIP1       PORTC,7        ;Layout Definition, Input (DIP1 -> ON = LOW = FREEZE / OFF = HIGH = kein FREEZE
#endif ;as88_2v6

#ifdef ms88
  #define IO_P_DATA     PORTA,0        ;Layout Definition, Input (Programming)
  #define IO_P_CLK      PORTA,1        ;Layout Definition, Input (Programming)
  #define IO_S88_CLK    PORTA,2        ;Layout Definition, Input IR (S88) 
  #define IO_VPP        PORTA,3        ;Layout Definition, Input (Input only) (Programming)
  #define IO_S88_PS     PORTA,4        ;Layout Definition, Input (S88)
  #define IO_DATA_IN    PORTA,5        ;Layout Definition, Input (S88)
  
  #define IO_RM4        R_PORTB,4      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DATA_OUT   PORTB,5        ;Layout Definition, Output (S88)
  #define IO_RM6        R_PORTB,6      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_LED_STATUS PORTB,7        ;Layout Definition, Output (LED)
  
  #define IO_RM1        R_PORTC,0      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM2        R_PORTC,1      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM3        R_PORTC,2      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM8        R_PORTC,3      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM5        R_PORTC,4      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DIP2       PORTC,5        ;Layout Definition, Input (DIP2 -> 1 = Inputs invertiert)
  #define IO_RM7        R_PORTC,6      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DIP1       PORTC,7        ;Layout Definition, Input (DIP1 -> 1 = FAST / 0 = SLOW Erkennung Inputs)
#endif ;ms88

#ifdef gls88
xx  #define IO_P_DATA     PORTA,0        ;Layout Definition, Input (Programming)
  #define IO_P_CLK      PORTA,1        ;Layout Definition, Input (Programming)
  #define IO_S88_CLK    PORTA,2        ;Layout Definition, Input IR (S88) 
  #define IO_VPP        PORTA,3        ;Layout Definition, Input (Input only) (Programming)
  #define IO_S88_PS     PORTA,4        ;Layout Definition, Input (S88)
  #define IO_DATA_IN    PORTA,5        ;Layout Definition, Input (S88)
  
  #define IO_RM2_B      R_PORTB,4      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DATA_OUT   PORTB,5        ;Layout Definition, Output (S88)
  #define IO_RM3_B      R_PORTB,6      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_LED_STATUS PORTB,7        ;Layout Definition, Output (LED)
  
  #define IO_RM1_A      R_PORTC,0      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM1_B      R_PORTC,1      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM2_A      R_PORTC,2      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM4_B      R_PORTC,3      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM3_A      R_PORTC,4      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DIP2       PORTC,5        ;Layout Definition, Input (DIP2 -> 1 = Inputs invertiert)
  #define IO_RM4_A      R_PORTC,6      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DIP1       PORTC,7        ;Layout Definition, Input (DIP1 -> 1 = FAST / 0 = SLOW Erkennung Inputs)
#endif ;gls88

#ifdef weich88
  #define IO_P_DATA     PORTA,0        ;Layout Definition, Input (Programming)
  #define IO_P_CLK      PORTA,1        ;Layout Definition, Input (Programming)
  #define IO_S88_CLK    PORTA,2        ;Layout Definition, Input IR (S88) 
  #define IO_VPP        PORTA,3        ;Layout Definition, Input (Input only) (Programming)
  #define IO_S88_PS     PORTA,4        ;Layout Definition, Input (S88)
  #define IO_DATA_IN    PORTA,5        ;Layout Definition, Input (S88)
  
  #define IO_RM2_B      R_PORTB,4      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_DATA_OUT   PORTB,5        ;Layout Definition, Output (S88)
  #define IO_RM3_B      R_PORTB,6      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_LED_STATUS PORTB,7        ;Layout Definition, Output (LED)
  
  #define IO_RM1_A      R_PORTC,0      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM1_B      R_PORTC,1      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM2_A      R_PORTC,2      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM4_B      R_PORTC,3      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RM3_A      R_PORTC,4      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RES2       PORTC,5        ;Layout Definition, Output (unused)
  #define IO_RM4_A      R_PORTC,6      ;Layout Definition, Input (wird aus LATCH Register geholt)
  #define IO_RES1       PORTC,7        ;Layout Definition, Output (unused)
#endif ;weich88


;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------

;   --------------------------------------------------------------------------
;   --- Timing-Parameter
;   --------------------------------------------------------------------------

#ifdef weich88
CNT_DEBOUNCE equ d'1'                  ; counter for debouncing: Weich88 - increase up to 255 for faster detection - decrease down to 1 for less interference liability
#endif

#ifdef as88
CNT_DEBOUNCE_ADD  equ d'100'           ; counter for debouncing AKTIV: MS88 - increase up to 255 for faster detection - decrease down to 1 for less interference liability
CNT_DEBOUNCE_SUB  equ d'1'             ; counter for debouncing INAKTIV: 
#endif

#ifdef as88_2v6
CNT_DEBOUNCE_ADD  equ d'100'           ; counter for debouncing AKTIV: MS88 - increase up to 255 for faster detection - decrease down to 1 for less interference liability
CNT_DEBOUNCE_SUB  equ d'1'             ; counter for debouncing INAKTIV: 
#endif ;as88_2v6

#ifdef ms88
CNT_DEBOUNCE_ADD  equ d'100'           ; counter for debouncing AKTIV: MS88 - increase up to 255 for faster detection - decrease down to 1 for less interference liability
CNT_DEBOUNCE_SUB  equ d'1'             ; counter for debouncing INAKTIV: 
CNT_PS_FREEZE_1   equ d'2'             ; counter, fr wie viele S88 Zyklen (=PS-Impulse) ein Bit mindestens AKTIV bleibt (Impuls-Verlngerung)
CNT_PS_FREEZE_2   equ d'10'            ; counter, fr wie viele S88 Zyklen (=PS-Impulse) ein Bit mindestens AKTIV bleibt (Impuls-Verlngerung)
#endif

#ifdef gls88
  CNT_DEBOUNCE equ d'150'              ; counter for debouncing: GLS88 (grer, da Lichtschranke) - increase up to 255 for faster detection - decrease down to 1 for less interference liability
  T_02    equ d'125'                   ; 32 ms
                                       ; Timer2 for IR periode (=repetition time)
                                       ; T_02= (Time in us)/(Prescaler x Postscaler)
#endif

CNT_S88                equ d'15'       ; counter for S88 recognizion (x 65 ms keine CLK-Flanke -> no S88 -> LED blinken)
CONST_S88_CLK_TIMEOUT  equ d'40'       ; (x 2,5 s) counter for S88 CLK Timeout


#ifdef dsd2010_grube
; DSD2010: Zum vereinfachten Ablauf die Byte-Anzahl wird ber die 1en im Register R_S88BIT_ANZAHL definiert.
grube_mode1_anz   equ b'00000000'      ; Anzahl der S88 Ausgangs-Bits in Mode 1 (DSD2010 - Grube) = 16 Bits (8 Status + 8 Pos.)
grube_mode2_anz   equ b'00000001'      ; Anzahl der S88 Ausgangs-Bits in Mode 2 (DSD2010 - Grube) = 24 Bits (8 Status + 8 Pos. + 8 Error)
grube_mode3_anz   equ b'00000010'      ; Anzahl der S88 Ausgangs-Bits in Mode 3 (DSD2010 - Grube) = 32 Bits (8 Status + 24 Pos.)
#ifdef dsd2010_grube_use_mode_5
grube_mode5_anz   equ b'00000100'      ; Anzahl der S88 Ausgangs-Bits in Mode 5 (DSD2010 - Grube) = 64 Bits (8 Status + 48 Pos. + 8 Error)
#else
;grube_mode4_anz   equ b'00011111'      ; Anzahl der S88 Ausgangs-Bits in Mode 4 (DSD2010 - Grube) = 40 Bits (8 Status + 24 Pos. + 8 Error)
#endif ; dsd2010_grube_use_mode_5
#endif ; dsd2010_grube

;grube_r_anz_bit   equ d'32'            ; Anzahl der Bits, die ber die Optokoppler bertragen werden

; ********************************************************
; **  Reset Vector                                      **
; ********************************************************

  org  0000h                           ; Position (Reset Vector)

; ********************************************************
; **  IDs for Manipulator                               **
; ********************************************************
  movlw d'250'
        goto start

; ********************************************************
; **  Interupt Service Routine ISR                      **
; ********************************************************
  org 0004h                            ; Interupt Vector
; ----- IR IB0-Change (negative/positive Edge) ------------


; ********************************************************
; ******* DATA-OUT bedienen                     **********
; ********************************************************
       ; erst einmal DATA-OUT bedienen (3 us nach CLK Rise - am 16f690-Pin gemessen)). W und STATUS not effected
;        bcf IO_DATA_OUT                ; .. (remember inverting transistor)

#ifdef test_s88
   bsf IO_test_1
#endif ; test_s88
#ifndef polarity_data_out_inverted
        btfss R_S88DATA_STREAM_0,0     ; Output of R_S88DATA_STREAM_0 Bit#0
         bcf IO_DATA_OUT               ; .. to DATA_OUT (inverted)
        btfsc R_S88DATA_STREAM_0,0     ; Output of R_S88DATA_STREAM_0 Bit#0
         bsf IO_DATA_OUT               ; .. to DATA_OUT (inverted)
#endif
#ifdef polarity_data_out_inverted
        btfsc R_S88DATA_STREAM_0,0     ; Output of R_S88DATA_STREAM_0 Bit#0
         bcf IO_DATA_OUT               ; .. (remember inverting transistor)
        btfss R_S88DATA_STREAM_0,0     ; Output of R_S88DATA_STREAM_0 Bit#0
         bsf IO_DATA_OUT               ; .. (remember inverting transistor)
#endif

; ********************************************************
; ******* Saving STATUS and W                   **********
; ********************************************************
       ; jetzt ist Zeit, die Register zu retten
        movwf W_TEMP                   ; W-Register Retten
        swapf STATUS,W                 ; Status-Reg. -> w
        movwf STATUS_TEMP              ; Status-Reg. Retten
        bcf STATUS,RP0                 ; -> BANK 0
        
; ********************************************************
; ******* Check if PS = activ                   **********
; ********************************************************
#ifndef polarity_ps_inverted
        btfss IO_S88_PS                ; if PS = HIGH
         goto s88_normal_2
#endif
#ifdef polarity_ps_inverted
        btfsc IO_S88_PS                ; if PS = LOW (=S88-HIGH)
         goto s88_normal_2
#endif

; ********************************************************
; ******* PS = activ  (10 us @ Grube - 8 Byte)  **********
; ********************************************************
        ; *** if PS = activ then load R_S88DATA_OUT into R_S88DATA_STREAM ***
        movf R_S88DATA_OUT_0,W         ; load input-values
         movwf R_S88DATA_STREAM_0      ; .. into R_S88DATA_STREAM_0

#ifdef dsd2010_grube ; nur DSD hat mehr als 8 Bits
        movf R_S88DATA_OUT_1,W         ; load input-values
         movwf R_S88DATA_STREAM_1      ; .. into R_S88DATA_STREAM_1
        movf R_S88DATA_OUT_2,W         ; load input-values
         movwf R_S88DATA_STREAM_2      ; .. into R_S88DATA_STREAM_2
        movf R_S88DATA_OUT_3,W         ; load input-values
         movwf R_S88DATA_STREAM_3      ; .. into R_S88DATA_STREAM_3
        movf R_S88DATA_OUT_4,W         ; load input-values
         movwf R_S88DATA_STREAM_4      ; .. into R_S88DATA_STREAM_4
        movf R_S88DATA_OUT_5,W         ; load input-values
         movwf R_S88DATA_STREAM_5      ; .. into R_S88DATA_STREAM_5
        movf R_S88DATA_OUT_6,W         ; load input-values
         movwf R_S88DATA_STREAM_6      ; .. into R_S88DATA_STREAM_6
        movf R_S88DATA_OUT_7,W         ; load input-values
         movwf R_S88DATA_STREAM_7      ; .. into R_S88DATA_STREAM_7
#endif ; dsd2010_grube

        ; *** Output Bit #0 to Data-Out ***
        ; dies ist nur dann interessant, wenn PS anliegt. Dann ist aber genug Zeit,
        ; .. um jetzt die "neuen" Daten noch auf den Bus zu legen.
#ifndef polarity_data_out_inverted
        btfss R_S88DATA_STREAM_0,0     ; Output of R_S88DATA_OUT_0 Bit#0
         bcf IO_DATA_OUT               ; .. 
        btfsc R_S88DATA_STREAM_0,0     ; .. 
         bsf IO_DATA_OUT               ; .. to DATA_OUT (inverted)
#endif
#ifdef polarity_data_out_inverted
        btfss R_S88DATA_STREAM_0,0     ; Output of R_S88DATA_OUT_0 Bit#0
         bsf IO_DATA_OUT               ; .. (remember inverting transistor)
        btfsc R_S88DATA_STREAM_0,0     ; .. 
         bcf IO_DATA_OUT               ; .. to DATA_OUT
#endif
        bsf F_PS_OK

; ********************************************************
; ******* warten auf die fallenden CLK-Flanke   **********
; ********************************************************
s88_normal_2
       ; warten auf die fallenden CLK-Flanke
       ; clrwdt  ; ntig? geht das in der ISR berhaupt? Knnte fr ECOS interessant sein,
                 ; ..da die ECOS sehr lange im CLK-HIGH Status ist (bis zu 1.2 ms)
#ifdef test_s88
   bsf IO_test_3
#endif ; test_s88

#ifndef polarity_clk_inverted
        btfss IO_S88_CLK               ; wait for CLK gets LOW
         goto s88_load_data            ; ... (negative Edge)
        decfsz R_S88_CLK_TIMEOUT,F
         goto $-3                      ;
#endif
#ifdef polarity_clk_inverted
        btfsc IO_S88_CLK               ; wait for CLK gets HIGH
         goto s88_load_data            ; ... (positive Edge)
        decfsz R_S88_CLK_TIMEOUT
         goto $-3                      ;
#endif

        ; jetzt Register laden und Flag lschen. schafft auch etwas Zeit bis zum Einlesen der Daten
s88_load_data
#ifdef test_s88
   bcf IO_test_3
#endif ; test_s88
        movlw CONST_S88_CLK_TIMEOUT
        movwf R_S88_CLK_TIMEOUT
        movlw CNT_S88
        movwf R_BUSERKENNUNG
        bcf INTCON,INTF                ; clear IR-Flag

; ********************************************************
; ******* Daten von DATA_IN einlesen            **********
; ********************************************************
        ; jetzt Daten einlesen so ca. 4-5 us nach der fallenden CLK-Flanke
        bsf STATUS,C                   ; Bit am Eingang DATA-IN zum

#ifndef polarity_data_in_inverted
        btfss IO_DATA_IN               ; .. Nachschieben in 
         bcf STATUS,C                  ; .. Schieberegister in C
#endif
#ifdef polarity_data_in_inverted
        btfsc IO_DATA_IN               ; .. Nachschieben in 
         bcf STATUS,C                  ; .. Schieberegister in C
#endif

; ************************************************************************
; ******* Daten schieben  (beendet 14/16 us nach CLK Fall @ 8 Byte *******
; ************************************************************************
        ; das C wird jetzt durch die Redister STREAM durchgeschoben (also hinten ins letzte Register "von links" hineingeschoben)
        ; dabei wird durch so viele Register geschoben, wie die Bitanzahl dieses Moduls es erfordert.
        ; hierfr wird zum vereinfachten Ablauf die Byte-Anzahl ber die 1en im Register R_S88BIT_ANZAHL definiert.

#ifdef dsd2010_grube ; nur DSD hat mehr als 8 Bits, bei allen anderen nur mit R_S88DATA_STREAM_0 arbeiten 
        btfsc R_S88BIT_ANZAHL,2        ; 64 bit
         goto s88_64bit
        btfsc R_S88BIT_ANZAHL,1        ; 32 bit
         goto s88_32bit
        btfsc R_S88BIT_ANZAHL,0        ; 16 bit
         goto s88_24bit
        goto s88_16bit
s88_64bit
#ifdef test_s88_bit
   bsf IO_test_2
   bcf IO_test_2
#endif ; test_s88
        rrf R_S88DATA_STREAM_7,F       ; Daten nachschieben
        rrf R_S88DATA_STREAM_6,F       ; Daten nachschieben
        rrf R_S88DATA_STREAM_5,F       ; Daten nachschieben
        rrf R_S88DATA_STREAM_4,F       ; Daten nachschieben
s88_32bit
#ifdef test_s88_bit
   bsf IO_test_2
   bcf IO_test_2
#endif ; test_s88
        rrf R_S88DATA_STREAM_3,F       ; Daten nachschieben
s88_24bit
#ifdef test_s88_bit
   bsf IO_test_2
   bcf IO_test_2
#endif ; test_s88
        rrf R_S88DATA_STREAM_2,F       ; Daten nachschieben
s88_16bit
#ifdef test_s88_bit
   bsf IO_test_2
   bcf IO_test_2
#endif ; test_s88
        rrf R_S88DATA_STREAM_1,F       ; Daten nachschieben (Grube immer mit mind. 2 Byte)
#endif

        rrf R_S88DATA_STREAM_0,F       ; Daten nachschieben (immer mind. 1 Byte)
      
#ifdef test_s88
   bcf IO_test_1
#endif ; test_s88

; ----- Restore STATUS and W ---------------------
        swapf STATUS_TEMP,W            ; in w-Register
        movwf STATUS                   ; w -> Status
        SWAPF W_TEMP,F                 ; W_TEMP -> W, ohne..
        SWAPF W_TEMP,W                 ; .. Z-Flag zu aendern

        retfie

; ********************************************************
; **  Programm                                          **
; ********************************************************
 org   0060h                           ; Memory-Position
;   -----------------------------
;   --- clear memory $20..$7f ---
;   -----------------------------
start
        movlw h'20'                    ; memory start adress
        movwf FSR
next_clr
        clrf INDF                      ; clear
        incf FSR,F
        movlw h'80'                    ; Anzahl der Adressen
        subwf FSR,W                    ; W = FSR - W = FSR - h'7F'
        btfss STATUS,C                 ; if C=0, -> FSR < h'7F'
         goto next_clr                  ; next bit

;   --------------------
;   --- start values ---
;   --------------------
        movlw CONST_S88_CLK_TIMEOUT
        movwf R_S88_CLK_TIMEOUT

#ifdef as88_2v6
        ; default-Wert fr 4094 KnightRider-Effekt
     ;   movlw d'1'
     ;   movwf R_4094_VALUES_1
     ;   bsf F_4094_MODE_KNIGHT
#endif ;as88_2v6

; ********************************************************
; **  Konfiguration PORTS                               **
; ********************************************************
#ifdef dsd2010_grube
        bsf STATUS,RP0                 ; -> BANK 1
          movlw b'11111111'            ; Port A: alles Input
          andwf TRISA,F
          movlw b'00011111'            ; Port B: #5 #6 #7 ist Output, Rest ist Input
          andwf TRISB,F
          movlw b'11000111'            ; Port C: #3 #4 #5 ist Output, Rest ist Input
          andwf TRISC,F
        bcf STATUS,RP0                 ; -> BANK 0
#endif ;dsd2010_grube

#ifdef as88
#ifndef test
        bsf STATUS,RP0                 ; -> BANK 1
          movlw b'11111110'            ; Port A: #0 ist Output, Rest alles Input
          andwf TRISA,F
          movlw b'01011111'            ; Port B: #5 #7 ist Output, Rest ist Input
          andwf TRISB,F
          movlw b'11111111'            ; Port C: alles ist Input
          andwf TRISC,F
        bcf STATUS,RP0                 ; -> BANK 0
#endif ; ifndef test
#ifdef test
        bsf STATUS,RP0                 ; -> BANK 1
          movlw b'11111100'            ; Port A: A0/1 OUTPUT fr TEST
          andwf TRISA,F
          movlw b'01011111'            ; Port B: #5 #7 ist Output, Rest ist Input
          andwf TRISB,F
          movlw b'11111111'            ; Port C: alles ist Input
          andwf TRISC,F
        bcf STATUS,RP0                 ; -> BANK 0
#endif ; test
#endif ;as88

#ifdef as88_2v6
#ifndef test
        bsf STATUS,RP0                 ; -> BANK 1
          movlw b'11111100'            ; Port A: #0,#1 ist Output, Rest alles Input
          andwf TRISA,F
          movlw b'01011111'            ; Port B: #5 #7 ist Output, Rest ist Input
          andwf TRISB,F
          movlw b'11111111'            ; Port C: alles ist Input
          andwf TRISC,F
        bcf STATUS,RP0                 ; -> BANK 0
#endif ; ifndef test
#ifdef test
        bsf STATUS,RP0                 ; -> BANK 1
          movlw b'11111100'            ; Port A: A0/1 OUTPUT fr TEST
          andwf TRISA,F
          movlw b'01011111'            ; Port B: #5 #7 ist Output, Rest ist Input
          andwf TRISB,F
          movlw b'11111111'            ; Port C: alles ist Input
          andwf TRISC,F
        bcf STATUS,RP0                 ; -> BANK 0
#endif ; test
#endif ;as88_2v6

#ifdef ms88
#ifndef test
        bsf STATUS,RP0                 ; -> BANK 1
          movlw b'11111111'            ; Port A: alles Input
          andwf TRISA,F
          movlw b'01011111'            ; Port B: #5 #7 ist Output, Rest ist Input
          andwf TRISB,F
          movlw b'11111111'            ; Port C: alles ist Input
          andwf TRISC,F
        bcf STATUS,RP0                 ; -> BANK 0

; WPUA: PORTA REGISTER (Pull Up Enable)
        bsf STATUS,RP0                 ; -> BANK 1
          movlw   B'00000011'          ; 1 = Pull-up enabled (A0, A1)
          movwf WPUA                   ; WPUS-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0
#endif ; ifndef test
#ifdef test
        bsf STATUS,RP0                 ; -> BANK 1
          movlw b'11111100'            ; Port A: A0/1 OUTPUT fr TEST
          andwf TRISA,F
          movlw b'01011111'            ; Port B: #5 #7 ist Output, Rest ist Input
          andwf TRISB,F
          movlw b'11111111'            ; Port C: alles ist Input
#ifdef test_use_DIP_as_OUT
          movlw b'01011111'            ; Port C: C5/c7 OUTPUT !! DIP muss OFF sein !!!
#endif ; test_use_DIP_as OUT
          andwf TRISC,F
        bcf STATUS,RP0                 ; -> BANK 0
#endif ; test
#endif ;ms88

#ifdef weich88
        bsf STATUS,RP0                 ; -> BANK 1
          movlw b'11111111'            ; Port A: alles Input
          andwf TRISA,F
          movlw b'01011111'            ; Port B: #5 #7 ist Output, Rest ist Input
          andwf TRISB,F
          movlw b'01011111'            ; Port C: #5 #7 ist Output, Rest ist Input
          andwf TRISC,F
        bcf STATUS,RP0                 ; -> BANK 0
#endif ;weich88

#ifdef gls88
   x toDo x     bsf STATUS,RP0                 ; -> BANK 1
          movlw b'11111111'            ; Port A: alles Input
          andwf TRISA,F
          movlw b'00011111'            ; Port B: #5 #6 #7 ist Output, Rest ist Input
          andwf TRISB,F
          movlw b'11000111'            ; Port C: #3 #4 #5 ist Output, Rest ist Input
          andwf TRISC,F
        bcf STATUS,RP0                 ; -> BANK 0
#endif ;gls88

        clrf PORTA
        clrf PORTB
        clrf PORTC

; <editor-fold  defaultstate="collapsed" desc="Konfiguration">
; ****************************************************************
; **  Konfiguration PCON: POWER CONTROL REGISTER (ADDRESS: 8Eh) **
; ****************************************************************
        movlw   B'00010000'
;                 ++-------- #7-#6: Unimplemented: Read as 0
;                   +------- #5: ULPWUE: Ultra Low Power Wake-up Enable bit
;                    |||||       1 = Ultra Low Power Wake-up enabled
;                    |||||      >0 = Ultra Low Power Wake-up disabled
;                    +------ #4: SBOREN: Software BOR Enable bit(1)
;                     ||||      >1 = BOR enabled                   
;                     ||||       0 = BOR disabled                  
;                     ++---- #3-#2: Unimplemented: Read as 0
;                       +--- #1: POR: Power-on Reset STATUS bit
;                        |       1 = No Power-on Reset occurred
;                        |      >0 = A Power-on Reset occurred (must be set in software after a Power-on Reset occurs)
;                        +-- #0: BOD: Brown-out Detect STATUS bit
;                                1 = No Brown-out Detect occurred
;                               >0 = A Brown-out Detect occurred 
;                                    (must be set in software after a Brown-out Detect occurs)

        bsf STATUS,RP0                 ; -> BANK 1
        movwf PCON                     ; PCON-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0

; ************************************************************************
; **  Konfiguration OSCCON: OSCILLATOR CONTROL REGISTER (ADDRESS: 8Fh)  **
; ************************************************************************
        movlw   B'01110000'
;                 +--------- #7: Unimplemented: Read as 0
;                  +++------ #6-#4: IRCF<2:0>: Internal Oscillator Frequency Select bits
;                     ||||       111 = 8MHz           
;                     ||||      >110 = 4MHz (default)
;                     ||||       101 = 2MHz           
;                     ||||       100 = 1MHz           
;                     ||||       011 = 500 kHz        
;                     ||||       010 = 250 kHz        
;                     ||||       001 = 125 kHz        
;                     ||||       000 = 31 kHz (LFINTOSC)
;                     +---   #3: OSTS: Oscillator Start-up Time-out STATUS bit
;                      |||       1 = Device is running from the primary system clock (FOSC<2:0>)
;                      |||       0 = Device is running from the secondary system clock (INTOSC or INTRC)
;                      +---- #2: HTS: INTOSC (High Frequency - 8 MHz to 125 kHz) STATUS bit
;                       ||       1 = INTOSC is stable
;                       ||      >0 = INTOSC is not stable
;                       +--- #1: LTS: LFINTOSC Stable bit (Low Frequency  31 kHz)
;                        |       1 = LFINTOSC is stable                           
;                        |      >0 = LFINTOSC is not stable                       
;                        +-- #0: SCS: Oscillator Mode Select bits
;                                1 = Internal oscillator is used for system clock
;                               >0 = Oscillator mode defined by FOSC<2:0> 

        bsf STATUS,RP0                 ; -> BANK 1
        movwf OSCCON                   ; OSCCON-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0

; ********************************************************
; **  Konfiguration COMPARATOR                          **
; ********************************************************
#ifndef dummy
        movlw   B'00000000'
;                 +--------- #7: C1ON: Comparator C1 Enable bit
;                  |||||||       1 = Comparator C1 is enabled
;                  |||||||     > 0 = Comparator C1 is disabled
;                  +-------- #6: C1OUT: Comparator C1 Output bit
;                   ||||||       If C1POL = 1 (inverted polarity):
;                   ||||||          C1OUT = 0 when C1VIN+ > C1VIN
;                   ||||||          C1OUT = 1 when C1VIN+ < C1VINIf
;                   ||||||       C1POL = 0 (non-inverted polarity):
;                   ||||||          C1OUT = 1 when C1VIN+ > C1VIN
;                   ||||||          C1OUT = 0 when C1VIN+ < C1VIN
;                   +------- #5: C1OE: Comparator C1 Output Enable bit
;                    |||||       1 = C1OUT is present on the C1OUT pin(1)
;                    |||||       0 = C1OUT is internal only
;                    +------ #4: C1POL: Comparator C1 Output Polarity Select bit
;                     ||||       1 = C1OUT logic is inverted    
;                     ||||       0 = C1OUT logic is not inverted
;                     +----- #3: Unimplemented: Read as 0
;                      +---- #2: C1R: Comparator C1 Reference Select bit (non-inverting input) 
;                       ||       1 = C1VIN+ connects to C1VREF output     
;                       ||       0 = C1VIN+ connects to C1IN+ pin         
;                       ++-- #1 #0: C1CH<1:0>: Comparator C1 Channel Select bit
;                                00 = C1VIN- of C1 connects to C12IN0- pin
;                                01 = C1VIN- of C1 connects to C12IN1- pin
;                                10 = C1VIN- of C1 connects to C12IN2- pin
;                                11 = C1VIN- of C1 connects to C12IN3- pin

        bsf STATUS,RP1             ; -> BANK 2
        movwf CM1CON0              ; CM1CON0-Reg. konfigurieren
        bcf STATUS,RP1             ; -> BANK 0
       
        movlw   B'00000000'
;                 +--------- #7: C2ON: Comparator C2 Enable bit
;                  |||||||       1 = Comparator C2 is enabled
;                  |||||||     > 0 = Comparator C2 is disabled
;                  +-------- #6: C2OUT: Comparator C2 Output bit
;                   ||||||       If C2POL = 1 (inverted polarity):
;                   ||||||       C2OUT = 0 when C2VIN+ > C2VIN
;                   ||||||       C2OUT = 1 when C2VIN+ < C2VINIf
;                   ||||||       C2POL = 0 (non-inverted polarity):
;                   ||||||       C2OUT = 1 when C2VIN+ > C2VIN
;                   ||||||       C2OUT = 0 when C2VIN+ < C2VIN
;                   +------- #5 C2OE: Comparator C2 Output Enable bit
;                    |||||       1 = C2OUT is present on C2OUT pin(1)
;                    |||||       0 = C2OUT is internal only
;                    +------ #4 C1POL: Comparator C1 Output Polarity Select bit
;                     ||||       1 = C1OUT logic is inverted
;                     ||||       0 = C1OUT logic is not inverted
;                     +----- #3 Unimplemented: Read as 0
;                      +---- #2 C2R: Comparator C2 Reference Select bits (non-inverting input)
;                       ||       1 = C2VIN+ connects to C2VREF
;                       ||       0 = C2VIN+ connects to C2IN+ pin
;                       ++-- #1-0 C2CH<1:0>: Comparator C2 Channel Select bits
;                                00 = C2VIN- of C2 connects to C12IN0- pin
;                                01 = C2VIN- of C2 connects to C12IN1- pin
;                                10 = C2VIN- of C2 connects to C12IN2- pin
;                                11 = C2VIN- of C2 connects to C12IN3- pin       

        bsf STATUS,RP1             ; -> BANK 2
        movwf CM2CON0              ; CM1CON0-Reg. konfigurieren
        bcf STATUS,RP1             ; -> BANK 0
#endif ;dummy

; ****************************************************************************
; **  Konfiguration CCP1CON: ENHANCED CCP1 CONTROL REGISTER  (ADDRESS: 17h) ** 
; ****************************************************************************
#ifndef dummy
        movlw   B'00000000'
;                 ++-------- #7-6: P1M<1:0>: PWM Output Configuration bits
;                   ||||||       If CCP1M<3:2> = 00, 01, 10:
;                   ||||||       xx = P1A assigned as Capture/Compare input; P1B, P1C, P1D assigned as port pins
;                   ||||||       If CCP1M<3:2> = 11:
;                   ||||||       00 = Single output; P1A modulated; P1B, P1C, P1D assigned as port pins
;                   ||||||       01 = Full-bridge output forward; P1D modulated; P1A active; P1B, P1C inactive
;                   ||||||       10 = Half-bridge output; P1A, P1B modulated with dead-band control; P1C, P1D assigned as port pins
;                   ||||||       11 = Full-bridge output reverse; P1B modulated; P1C active; P1A, P1D inactive
;                   ++------ #5-4: DC1B<1:0>: PWM Duty Cycle Least Significant bits
;                     ||||       Capture mode: Unused.
;                     ||||       Compare mode: Unused.
;                     ||||       PWM mode: These bits are the two LSbs of the PWM duty cycle. The eight MSbs are found in CCPR1L.
;                     ++++--- #3-0:  CCP1M<3:0>: ECCP Mode Select bits
;                               >0000 = Capture/Compare/PWM off (resets ECCP module)
;                                0001 = Unused (reserved)
;                                0010 = Compare mode, toggle output on match (CCP1IF bit is set)
;                                0011 = Unused (reserved)
;                                0100 = Capture mode, every falling edge
;                                0101 = Capture mode, every rising edge
;                                0110 = Capture mode, every 4th rising edge
;                                0111 = Capture mode, every 16th rising edge
;                                1000 = Compare mode, set output on match (CCP1IF bit is set)
;                                1001 = Compare mode, clear output on match (CCP1IF bit is set)
;                                1010 = Compare mode, generate software interrupt on match (CCP1IF bit is set, CCP1 pin is
;                                unaffected)
;                                1011 = Compare mode, trigger special event (CCP1IF bit is set; CCP1 resets TMR1 or TMR2, and starts
;                                an A/D conversion, if the ADC module is enabled)
;                                1100 = PWM mode; P1A, P1C active-high; P1B, P1D active-high
;                                1101 = PWM mode; P1A, P1C active-high; P1B, P1D active-low
;                                1110 = PWM mode; P1A, P1C active-low; P1B, P1D active-high
;                                1111 = PWM mode; P1A, P1C active-low; P1B, P1D active-low

        movwf CCP1CON                    ; CCP1CON:-Reg. konfigurieren
#endif ;dummy

; ********************************************************
; **  Konfiguration AD-Ports                            **
; ********************************************************
#ifndef dummy
        movlw   B'00000000'
;                 +--------- #7: ADFM: A/D Result Formed Select bit
;                  |||||||       1 = Right justified
;                  |||||||      >0 = Left justified
;                  +-------- #6: VCFG: Voltage Reference bit
;                   ||||||       1 = VREF pin
;                   ||||||      >0 = VDD
;                   ++++---- #5-#2: CHS1:CHS0: Analog Channel Select bits
;                       ||       0000 = AN0
;                       ||       0001 = AN1
;                       ||       0010 = AN2
;                       ||       0011 = AN3
;                       ||       0100 = AN4
;                       ||       0101 = AN5
;                       ||       0110 = AN6
;                       ||       0111 = AN7
;                       ||       1000 = AN8
;                       ||       1001 = AN9
;                       ||       1010 = AN10
;                       ||       1011 = AN11
;                       ||       1100 = CVREF
;                       ||       1101 = 0.6V Fixed Voltage Reference
;                       ||       1110 = Reserved. Do not use.
;                       ||       1111 = Reserved. Do not use.
;                       +--- #1: GO/DONE: A/D Conversion Status bit
;                        |       1 = A/D conversion cycle in progress. Setting this bit starts an 
;                        |           A/D conversion cycle.
;                        |           This bit is automatically cleared by hardware when 
;                        |           the A/D conversion has completed.
;                        |      >0 = A/D conversion completed/not in progress
;                        +-- #0: ADON: A/D Conversion Status bit
;                                1 = ADC is enabled
;                               >0 = ADC is disabled and consumes no operating current

        movwf ADCON0                   ; ADCON0-Reg. konfigurieren

        movlw   B'00000000'
;                 +--------- #7: Unimplemented: Read as 0.
;                  +++------ #6-#4: ADCS<2:0>: A/D Conversion Clock Select bits
;                     ||||      >000 = FOSC/2
;                     ||||       001 = FOSC/8 (-> 2.0 us, minimum is 1.6 us)
;                     ||||       010 = FOSC/32
;                     ||||       x11 = FRC (clock derived from a dedicated internal oscillator = 500 kHz max)
;                     ||||       100 = FOSC/4
;                     ||||       101 = FOSC/16
;                     ||||       110 = FOSC/64
;                     ++++-- #3-#0: Unimplemented: Read as 0

        bsf STATUS,RP0                 ; -> BANK 1
        movwf ADCON1                   ; ADCON1-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0

        movlw   B'00000000'
;                 ++++++++-- #7-#0: ANS<7:0>: Analog Select bits                                                   
;                                Analog select between analog or digital function on pins AN<7:0>, respectively.
;                                1 = Analog input. Pin is assigned as analog input(1).                          
;                                0 = Digital I/O. Pin is assigned to port or special function.                  
;                                    #7 = ANS7: Digital 
;                                    #6 = ANS6: Digital
;                                    #5 = ANS5: Digital
;                                    #4 = ANS4: Digital
;                                    #3 = ANS3: Digital
;                                    #2 = ANS2: Digital
;                                    #1 = ANS1: Digital
;                                    #0 = ANS0: Digital

        bsf STATUS,RP1                 ; -> BANK 2
        movwf ANSEL                    ; ANSEL-Reg. konfigurieren
        bcf STATUS,RP1                 ; -> BANK 0

        movlw   B'00000000'
;                     ++++-- #3-#0: ANS<11:8>: Analog Select bits                                                   
;                                Analog select between analog or digital function on pins AN<7:0>, respectively.
;                                1 = Analog input. Pin is assigned as analog input(1).                          
;                                0 = Digital I/O. Pin is assigned to port or special function.                  
;                                    #3 = ANS11: Digital
;                                    #2 = ANS10: Digital
;                                    #1 = ANS9: Digital
;                                    #0 = ANS8: Digital

        bsf STATUS,RP1                 ; -> BANK 2
        movwf ANSELH                   ; ANSELH-Reg. konfigurieren
        bcf STATUS,RP1                 ; -> BANK 0
#endif ;dummy

; ********************************************************
; **  Konfiguration TIMER / WDT / Pull-Ups              **
; ********************************************************
#ifndef dummy
#ifndef polarity_clk_inverted
        movlw   B'11000111'
#endif
#ifdef polarity_clk_inverted
        movlw   B'10000111'
#endif
;                 +--------- #7: RABPU: PORTA/PORTB Pull-up Enable bit
;                  |||||||      >1 = PORTA/PORTB pull-ups are disabled
;                  |||||||       0 = PORTA/PORTB pull-ups are enabled by individual PORT latch values
;                  +-------- #6: INTEDG: Interrupt Edge Select bit          
;                   ||||||      >1 = Interrupt on rising edge of RA2/INT pin
;                   ||||||       0 = Interrupt on falling edge of RA2/INT pin
;                   +------- #5: T0CS: Timer0 Clock Source Select bit
;                    |||||       1 = Transition on RA2/T0CKI pin     
;                    |||||      >0 = Internal instruction cycle clock (FOSC/4)
;                    +------ #4: T0SE: Timer0 Source Edge Select bit                     
;                     ||||       1 = Increment on high-to-low transition on RA2/T0CKI pin
;                     ||||      >0 = Increment on low-to-high transition on RA2/T0CKI pin
;                     +----- #3: PSA: Prescaler Assignment bit
;                      |||       1 = Prescaler is assigned to the WDT
;                      |||      >0 = Prescaler is assigned to the TIMER0 module
;                      +++-- #2-#0: PS2:PS0: Prescaler Rate Select bits
;                                 Bit |TMR0 Rate|WDT Rate
;                                 ----+---------+--------
;                                 000 | 1 : 2   | 1 : 1
;                                 001 | 1 : 4   | 1 : 2
;                                 010 | 1 : 8   | 1 : 4
;                                 011 | 1 : 16  | 1 : 8
;                                 100 | 1 : 32  | 1 : 16
;                                 101 | 1 : 64  | 1 : 32
;                                 110 | 1 : 128 | 1 : 64
;                               > 111 | 1 : 256 | 1 : 128

        bsf STATUS,RP0                 ; -> BANK 1
        movwf OPTION_REG               ; OPTION-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0

        movlw   B'00110000'
;                 +--------- #7: T1GINV: Timer1 Gate Invert bit(1)                                             
;                  |||||||       1 = Timer1 gate is active high (Timer1 counts when Timer1 gate signal is high)
;                  |||||||      >0 = Timer1 gate is active low (Timer1 counts when gate is low)                
;                  +-------- #6: TMR1GE: Timer1 Gate Enable bit
;                   ||||||      >If TMR1ON = 0: This bit is ignored
;                   ||||||       If TMR1ON = 1:
;                   ||||||          1 = Timer1 is on if T1G pin is low
;                   ||||||          0 = Timer1 is on
;                   ++------ #5-#4: T1CKPS1:T1CKPS0: Timer1 Input Clock Prescale Select bits
;                    |||||      >11 = 1:8 Prescale Value
;                    |||||       10 = 1:4 Prescale Value
;                    |||||       01 = 1:2 Prescale Valueue
;                    |||||       00 = 1:1 Prescale Value
;                     +----- #3: T1OSCEN: LP Oscillator Enable Control bit
;                      |||       If INTOSC without CLKOUT oscillator is active:
;                      |||       1 = LP oscillator is enabled for Timer1 clock
;                      |||      >0 = LP oscillator is off
;                      |||       Else: This bit is ignored
;                      +---- #2: T1SYNC: Timer1 External Clock Input Synchronization Control bit
;                       ||       TMR1CS = 1:                                
;                       ||          1 = Do not synchronize external clock input
;                       ||          0 = Synchronize external clock input       
;                       ||      >if TMR1CS = 0: This bit is ignored. Timer1 uses the internal clock.
;                       +--- #1: TMR1CS: Timer1 Clock Source Select bit
;                        |       1 = External clock from T1OSO/T1CKI pin (on the rising edge)
;                        |      >0 = Internal clock (FOSC/4)
;                        +-- #0: TMR1ON: Timer1 On bit
;                                1 = Enables Timer1
;                              > 0 = Stops Timer1

        movwf T1CON                    ; T1CON-Reg. konfigurieren

        movlw   B'00000000'
;                 +--------- #7: Unimplemented: Read as 0                                           
;                  ++++----- #6-#3: TOUTPS<3:0>: Timer2 Output Postscaler Select bits
;                      |||      >0000 = 1:1 Postscaler                            
;                      |||       0001 = 1:2 Postscaler                            
;                      |||       0010 = 1:3 Postscaler                            
;                      |||       0011 = 1:4 Postscaler                            
;                      |||       0100 = 1:5 Postscaler                            
;                      |||       0101 = 1:6 Postscaler                            
;                      |||       0110 = 1:7 Postscaler                            
;                      |||       0111 = 1:8 Postscaler                            
;                      |||       1000 = 1:9 Postscaler                            
;                      |||       1001 = 1:10 Postscaler                           
;                      |||       1010 = 1:11 Postscaler                           
;                      |||       1011 = 1:12 Postscaler                           
;                      |||       1100 = 1:13 Postscaler                           
;                      |||       1101 = 1:14 Postscaler                           
;                      |||       1110 = 1:15 Postscaler                           
;                      |||       1111 = 1:16 Postscaler                           
;                      +---- #2: TMR2ON: Timer2 On bit
;                       ||       1 = Timer2 is on     
;                       ||      >0 = Timer2 is off    
;                       ++-- #1-#0: T2CKPS<1:0>: Timer2 Clock Prescale Select bits
;                               >00 = Prescaler is 1                           
;                                01 = Prescaler is 4                           
;                                1x = Prescaler is 16                          

        movwf T2CON                    ; T2CON-Reg. konfigurieren

#endif ;dummy

; ********************************************************
; **  Konfiguration SPI                                 **
; ********************************************************
        movlw   B'00000000'
;                 +--------- #7 SMP: SPI Data Input Sample Phase bit
;                  |||||||      SPI Master mode:
;                  |||||||      1 = Input data sampled at end of data output time
;                  |||||||      0 = Input data sampled at middle of data output time (Microwire)
;                  |||||||      SPI Slave mode: SMP must be cleared when SPI is used in Slave mode
;                  |||||||      I2 C mode: This bit must be maintained clear
;                  +-------- #6 CKE: SPI Clock Edge Select bit
;                   ||||||      SPI mode, CKP = 0:
;                   ||||||      1 = Data transmitted on rising edge of SCK (Microwire alternate)
;                   ||||||      0 = Data transmitted on falling edge of SCK
;                   ||||||      SPI mode, CKP = 1:
;                   ||||||      1 = Data transmitted on falling edge of SCK (Microwire default)
;                   ||||||      0 = Data transmitted on rising edge of SCK
;                   ||||||      I2 C mode: This bit must be maintained clear
;                   +------- #5 D/A: Data/Address bit (I2C mode only)
;                    |||||      1 = Indicates that the last byte received or transmitted was data
;                    |||||      0 = Indicates that the last byte received or transmitted was address
;                    +------ #4 P: Stop bit (I2C mode only)
;                     ||||      This bit is cleared when the SSP module is disabled, or when the Start bit is detected last.
;                     ||||      SSPEN is cleared.
;                     ||||      1 = Indicates that a Stop bit has been detected last (this bit is 0 on Reset)
;                     ||||      0 = Stop bit was not detected last
;                     +----- #3 S: Start bit (I2C mode only)
;                      |||      This bit is cleared when the SSP module is disabled, or when the Stop bit is detected last.
;                      |||      SSPEN is cleared.
;                      |||      1 = Indicates that a Start bit has been detected last (this bit is 0 on Reset)
;                      |||      0 = Start bit was not detected last
;                      +---- #2 R/W: Read/Write bit Information (I2C mode only)
;                       ||      This bit holds the R/W bit information following the last address match. This bit is only valid from the
;                       ||      address match to the next Start bit, Stop bit or ACK bit.
;                       ||      1 = Read
;                       ||      0 = Write
;                       +--- #1 UA: Update Address bit (10-bit I2C mode only)
;                        |      1 = Indicates that the user needs to update the address in the SSPADD register
;                        |      0 = Address does not need to be updated
;                        +-- #0 BF: Buffer Full Status bit
;                               Receive (SPI and I2 C modes):
;                               1 = Receive complete, SSPBUF is full
;                               0 = Receive not complete, SSPBUF is empty
;                               Transmit (I2 C mode only):
;                               1 = Transmit in progress, SSPBUF is full
;                               0 = Transmit complete, SSPBUF is empty

        bsf STATUS,RP0                 ; -> BANK 1
        movwf SSPSTAT                  ; SSPSTAT-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0

        movlw   B'00000000'
;                 +--------- #7 WCOL: Write Collision Detect bit                                                                                 
;                  |||||||      1 = The SSPBUF register is written while it is still transmitting the previous word (must be cleared in software)
;                  |||||||      0 = No collision                                                                                                 
;                  |||||||      
;                  |||||||      
;                  |||||||      
;                  +-------- #6 SSPOV: Receive Overflow Indicator bit                                                                        
;                   ||||||      In SPI mode:                                                                                                
;                   ||||||      1 = A new byte is received while the SSPBUF register is still holding the previous data. In case of overflow,
;                   ||||||      the data in SSPSR is lost. Overflow can only occur in Slave mode. The user must read the SSPBUF,             
;                   ||||||      even if only transmitting data, to avoid setting overflow. In Master mode, the overflow bit is not set       
;                   ||||||      since each new reception (and transmission) is initiated by writing to the SSPBUF register.                  
;                   ||||||      0 = No overflow                                                                                              
;                   ||||||      In I2 C mode:                                                                                               
;                   ||||||      1 = A byte is received while the SSPBUF register is still holding the previous byte. SSPOV is a dont care 
;                   ||||||      in Transmit mode. SSPOV must be cleared in software in either mode.                                          
;                   ||||||      0 = No overflow                                                                                              
;                   +------- #5 SSPEN: Synchronous Serial Port Enable bit                                              
;                    |||||      In SPI mode:                                                                           
;                    |||||      1 = Enables serial port and configures SCK, SDO and SDI as serial port pins            
;                    |||||      0 = Disables serial port and configures these pins as I/O port pins                    
;                    |||||      In I2 C mode:                                                                          
;                    |||||      1 = Enables the serial port and configures the SDA and SCL pins as serial port pins    
;                    |||||      0 = Disables serial port and configures these pins as I/O port pins                    
;                    |||||      In both modes, when enabled, these pins must be properly configured as input or output.
;                    +------ #4 CKP: Clock Polarity Select bit                                        
;                     ||||      In SPI mode:                                                          ast.
;                     ||||      1 = Idle state for clock is a high level (Microwire default)          
;                     ||||      0 = Idle state for clock is a low level (Microwire alternate)         
;                     ||||      In I2 C mode:                                                         
;                     ||||      SCK release control                                                   
;                     ||||      1 = Enable clock                                                      
;                     ||||      0 = Holds clock low (clock stretch). (Used to ensure data setup time.)
;                     ++++--- #3-#0 SSPM<3:0>: Synchronous Serial Port Mode Select bits                                        
;                             0000 = SPI Master mode, clock = FOSC/4                                                     
;                             0001 = SPI Master mode, clock = FOSC/16                                                    
;                             0010 = SPI Master mode, clock = FOSC/64                                                    
;                             0011 = SPI Master mode, clock = TMR2 output/2                                              
;                             0100 = SPI Slave mode, clock = SCK pin. SS pin control enabled.                            
;                             0101 = SPI Slave mode, clock = SCK pin. SS pin control disabled. SS can be used as I/O pin.
;                             0110 = I2C Slave mode, 7-bit address                                                       
;                             0111 = I2C Slave mode, 10-bit address                                                      
;                             1000 = Reserved                                                                            
;                             1001 = Load SSPMSK register at SSPADD SFR address(2)                                       
;                             1010 = Reserved                                                                            
;                             1011 = I2C Firmware Controlled Master mode (slave IDLE)                                    
;                             1100 = Reserved                                                                            
;                             1101 = Reserved                                                                            
;                             1110 = I2C Slave mode, 7-bit address with Start and Stop bit interrupts enabled            
;                             1111 = I2C Slave mode, 10-bit address with Start and Stop bit interrupts enabled           
;                              
        movwf SSPCON                 ; SSPCON-Reg. konfigurieren


; ********************************************************
; **  Konfiguration PWM                                 **
; ********************************************************

; ********************************************************
; **  Konfiguration USART                               **
; ********************************************************
#ifndef dummy
        movlw   B'10100110'
;                 +--------- #7: CSRC: Clock Source Select bit
;                  |||||||       Asynchronous mode:                                   
;                  |||||||       Dont care                                           
;                  |||||||       Synchronous mode:                                    
;                  |||||||      >1 = Master mode (clock generated internally from BRG)
;                  |||||||       0 = Slave mode (clock from external source)          
;                  +-------- #6: TX9: 9-bit Transmit Enable bit
;                   ||||||       1 = Selects 9-bit transmission
;                   ||||||      >0 = Selects 8-bit transmission
;                   +------- #5: TXEN: Transmit Enable bit(1)
;                    |||||      >1 = Transmit enabled        
;                    |||||       0 = Transmit disabled       
;                    +------ #4: SYNC: EUSART Mode Select bit
;                     ||||       1 = Synchronous mode        
;                     ||||      >0 = Asynchronous mode       
;                     +----- #3: SENDB: Send Break Character bit                                               
;                      |||       Asynchronous mode:                                                            
;                      |||       1 = Send Sync Break on next transmission (cleared by hardware upon completion)
;                      |||      >0 = Sync Break transmission completed                                         
;                      |||       Synchronous mode:                                                             
;                      |||       Dont care                                                                    
;                      +---- #2: BRGH: High Baud Rate Select bit
;                       ||       Asynchronous mode:             
;                       ||      >1 = High speed                 
;                       ||       0 = Low speed                  
;                       ||       Synchronous mode:              
;                       ||       Unused in this mode            
;                       +--- #1: TRMT: Transmit Shift Register Status bit 
;                        |      >1 = TSR empty                           
;                        |       0 = TSR full                            
;                        +-- #0: TX9D: Ninth bit of Transmit Data
;                                Can be address/data bit or a parity bit.



        bsf STATUS,RP0                 ; -> BANK 1
        movwf TXSTA
        bcf STATUS,RP0                 ; -> BANK 0

        movlw   B'00000000'
;                 +--------- #7: ABDOVF: Auto-Baud Detect Overflow bit
;                  |||||||       Asynchronous mode:                   
;                  |||||||       1 = Auto-baud timer overflowed       
;                  |||||||       0 = Auto-baud timer did not overflow 
;                  |||||||       Synchronous mode:                    
;                  +-------- #6: RCIDL: Receive Idle Flag bit                                 
;                   ||||||       Asynchronous mode:                                           
;                   ||||||       1 = Receiver is Idle                                         
;                   ||||||       0 = Start bit has been received and the receiver is receiving
;                   ||||||       Synchronous mode:                                            
;                   ||||||       Dont care                                                   
;                   +------- #5: Unimplemented: Read as 0                
;                    +------ #4: SCKP: Synchronous Clock Polarity Select bit        
;                     ||||       Asynchronous mode:                                 
;                     ||||       1 = Transmit inverted data to the RB7/TX/CK pin                 
;                     ||||       0 = Transmit non-inverted data to the RB7/TX/CK pin             
;                     ||||       Synchronous mode:                                               
;                     ||||       1 = Data is clocked on rising edge of the clock                 
;                     ||||       0 = Data is clocked on falling edge of the clock   
;                     +----- #3: BRG16: 16-bit Baud Rate Generator bit 
;                      |||       1 = 16-bit Baud Rate Generator is used
;                      |||       0 = 8-bit Baud Rate Generator is used 
;                      +---- #2: Unimplemented: Read as 0                                                                
;                       +--- #1: WUE: Wake-up Enable bit                                                                                  
;                        |       Asynchronous mode:                                                                                       
;                        |       1 = Receiver is waiting for a falling edge. No character will be received byte RCIF will be set. WUE will
;                        |       automatically clear after RCIF is set.                                                                   
;                        |       0 = Receiver is operating normally                                                                       
;                        |       Synchronous mode:                                                                                        
;                        |       Dont care                                                                                               
;                        +-- #0: ABDEN: Auto-Baud Detect Enable bit                                      
;                                Asynchronous mode:                                                      
;                                1 = Auto-Baud Detect mode is enabled (clears when auto-baud is complete)
;                                0 = Auto-Baud Detect mode is disabled                                   
;                                Synchronous mode:                                                       
;                                Dont care                                                              

        bsf STATUS,RP0                 ; -> BANK 1
        movwf BAUDCTL
        bcf STATUS,RP0                 ; -> BANK 0

        movlw   B'00000000'
;                 +--------- #7: SPEN: Serial Port Enable bit
;                  |||||||       1 = Serial port enabled (configures RX/DT and TX/CK pins as serial port pins)
;                  |||||||      >0 = Serial port disabled (held in Reset)                                     
;                  +-------- #6: RX9: 9-bit Receive Enable bit
;                   ||||||       1 = Selects 9-bit reception  
;                   ||||||      >0 = Selects 8-bit reception  
;                   +------- #5: SREN: Single Receive Enable bit                 
;                    |||||       Asynchronous mode:                              
;                    |||||       Dont care                                      
;                    |||||       Synchronous mode  Master:                      
;                    |||||       1 = Enables single receive                      
;                    |||||      >0 = Disables single receive                     
;                    |||||       This bit is cleared after reception is complete.
;                    |||||       Synchronous mode  Slave                        
;                    |||||       Dont care                                      
;                    +------ #4: CREN: Continuous Receive Enable bit                                                  
;                     ||||       Asynchronous mode:                                                                   
;                     ||||       1 = Enables receiver                                                                              
;                     ||||      >0 = Disables receiver                                                                             
;                     ||||       Synchronous mode:                                                                                 
;                     ||||       1 = Enables continuous receive until enable bit CREN is cleared (CREN overrides SREN)             
;                     ||||       0 = Disables continuous receive                                                                   
;                     +----- #3: ADDEN: Address Detect Enable bit                                                              
;                      |||       Asynchronous mode 9-bit (RX9 = 1):                                                            
;                      |||       1 = Enables address detection, enable interrupt and load the receive buffer when RSR<8> is set
;                      |||      >0 = Disables address detection, all bytes are received and ninth bit can be used as parity bit
;                      |||       Asynchronous mode 8-bit (RX9 = 0):                                                            
;                      |||       Dont care                                                                                    
;                      +---- #2: FERR: Framing Error bit                                                                 
;                       ||       1 = Framing error (can be updated by reading RCREG register and receive next valid byte)
;                       ||       0 = No framing error                                                                    
;                       +--- #1: OERR: Overrun Error bit                                
;                        |       1 = Overrun error (can be cleared by clearing bit CREN)
;                        |       0 = No overrun error                                   
;                        +-- #0: RX9D: Ninth bit of Received Data
;                                This can be address/data bit or a parity bit and must be calculated by user firmware.
        movwf RCSTA

;        movlw   d'12'          ; 19  kBaud @ 4 MHz, Tabelle S. 165 (SYNC = 0, BRGH = 1, BRG16 = 0)
;        movlw   d'25'          ; 9.6 kBaud @ 4 MHz, Tabelle S. 165 (SYNC = 0, BRGH = 1, BRG16 = 0)
;        movlw   d'25'         ;  19  kBaud @ 8 MHz, Tabelle S. 164 (SYNC = 0, BRGH = 1, BRG16 = 0)
        movlw   d'51'          ;  9.6 kBaud @ 8 MHz, Tabelle S. 164 (SYNC = 0, BRGH = 1, BRG16 = 0)

                               ; 26 us / Bit = 234 us / Byte -> 500 ms fr Byte & Pause
                               ; DCC Fzg-Befehle IBox min alle 12 ms, meist alle 35 ms 
                               ; d.h. max. 24 Bytes!
                               ; SPBRG = (4MHz/(BAUD*16))-1
        bsf STATUS,RP0                 ; -> BANK 1
        movwf SPBRG
        bcf STATUS,RP0                 ; -> BANK 0
#endif ;dummy

; ********************************************************
; **  Konfiguration INTERRUPT                           **
; ********************************************************
#ifndef dummy
        movlw   B'00000000'
;                 +--------- #7: Unimplemented: Read as 0
;                  +-------- #6: ADIE: A/D Converter (ADC) Interrupt Enable bit(5)
;                                1 = Enables the ADC interrupt                    
;                               >0 = Disables the ADC interrupt                   
;                   +------- #5: RCIE: EUSART Receive Interrupt Enable bit(3)
;                                1 = Enables the EUSART receive interrupt    
;                                0 = Disables the EUSART receive interrupt   
;                    +------ #4: TXIE: EUSART Transmit Interrupt Enable bit(5)
;                                1 = Enables the EUSART transmit interrupt    
;                                0 = Disables the EUSART transmit interrupt   
;                     +----- #3: SSPIE: Synchronous Serial Port (SSP) Interrupt Enable bit(4)
;                                1 = Enables the SSP interrupt                               
;                               >0 = Disables the SSP interrupt                              
;                      +---- #2: CCP1IE: CCP1 Interrupt Enable bit(2)
;                                1 = Enables the CCP1 interrupt      
;                                0 = Disables the CCP1 interrupt     
;                       +--- #1: TMR2IE: Timer2 to PR2 Match Interrupt Enable bit(1)
;                                1 = Enables the Timer2 to PR2 match interrupt      
;                                0 = Disables the Timer2 to PR2 match interrupt     
;                        +-- #0: TMR1IE: Timer1 Overflow Interrupt Enable bit
;                                1 = Enables the Timer1 overflow interrupt   
;                               >0 = Disables the Timer1 overflow interrupt  

        bsf STATUS,RP0                 ; -> BANK 1
        movwf PIE1                     ; PIE1-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0

        movlw   B'00000000'
;                 +--------- #7: OSFIE: Oscillator Fail Interrupt Enable bit
;                                1 = Enables oscillator fail interrupt      
;                                0 = Disables oscillator fail interrupt     
;                  +-------- #6: C2IE: Comparator C2 Interrupt Enable bit
;                                1 = Enables Comparator C2 interrupt     
;                               >0 = Disables Comparator C2 interrupt    
;                   +------- #5: C1IE: Comparator C1 Interrupt Enable bit
;                                1 = Enables Comparator C1 interrupt     
;                                0 = Disables Comparator C1 interrupt    
;                    +------ #4: EEIE: EE Write Operation Interrupt Enable bit
;                                1 = Enables write operation interrupt        
;                                0 = Disables write operation interrupt       
;                     +----- #3-#0: Unimplemented: Read as 0

        bsf STATUS,RP0                 ; -> BANK 1
        movwf PIE2                     ; PIE2-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0

        movlw   B'00010000'
;                 +--------- #7: GIE: Global Interrupt Enable bit
;                  |||||||       1 = Enables all un-masked interrupts
;                  |||||||      >0 = Disables all interrupts, enable later
;                  +-------- #6: PEIE: Peripherial Interrupt Enable bit
;                   ||||||       1 = Enables all unmasked peripherial interrupts
;                   ||||||      >0 = Disables all peripherial interrupts
;                   +------- #5: T0IE: TMR0 Overflow Interrupt Enable bit
;                    |||||       1 = Enables the TMR0 interrupt
;                    |||||      >0 = Disables the TMR0 interrupt
;                    +------ #4: INTE: GP2/INT External Interrupt Enable bit
;                     ||||      >1 = Enables the GP2/INT external interrupt
;                     ||||       0 = Disables the GP2/INT external interrupt
;                     +----- #3: RABIE: PORTA/PORTB Change Interrupt Enable bit(1,3)
;                      |||       1 = Enables the PORTA/PORTB change interrupt       
;                      |||      >0 = Disables the PORTA/PORTB change interrupt      
;                      +---- #2: T0IF: TMR0 overflow interrupt flag bit
;                       ||       1 = TMR0 has overflowed (must be cleared in software)
;                       ||      >0 = TMR0 did not overflow
;                       +--- #1: INTF: GP2/INT External Interrupt Flag bit
;                        |       1 = The GP2/INT external interrupt occurred (must be cleared in software)
;                        |      >0 = The GP2/INT external interrupt did not occur
;                        +-- #0: RABIF: PORTA/PORTB Change Interrupt Flag bit                                               
;                                1 = When at least one of the PORTA or PORTB general purpose 
;                                    I/O pins changed state (must be cleared in software)                                                                       
;                               >0 = None of the PORTA or PORTB general purpose I/O pins have changed state                 

        movwf INTCON                   ; INTCON-Reg. konfigurieren
#endif; dummy
; </editor-fold>

; ********************************************************
; ********************************************************
; ********************************************************
; **  read EE-Data                                      **
; ********************************************************
; ********************************************************
; ********************************************************


; ********************************************************
; ********************************************************
; ********************************************************
; **  Wait                                              **
; ********************************************************
; ********************************************************
; ********************************************************
        decfsz R_TEMP,F                ; wait for Bus stabilisation (~500 us)
         goto $-1                      ; (wenn PS-Signal verzgert kommt, kann falsche Polaritt erkannt werden.

; <editor-fold  defaultstate="collapsed" desc="Initialisierung">
; ********************************************************
; ********************************************************
; ********************************************************
; **  Initialisierung                                   **
; ********************************************************
; ********************************************************
; ********************************************************
        movlw d'1'
        movwf R_TMR0_CNT
#ifdef as88_2v6
        movlw d'2'
        movwf R_TIMECOUNT_2;
#endif ;as88_2v6

        ; ------------------------------------
        ; --- Timer rcksetzten, IR Scharf ---
        ; ------------------------------------
        clrf TMR0
        clrf TMR1L                     ; clear Timer1 for Time-Out
        clrf TMR1H                     ; clear Timer1 for Time-Out

        bcf INTCON,T0IF                ; clear Flag
        bcf INTCON,INTF                ; clear IR-Flag
        bcf INTCON,RABIF               ; clear IR-Flag
        bcf PIR1,RCIF                  ; clear IR-Flag
        clrf PIR1                      ; clear all IR flags
        clrf PIR2                      ; clear all IR flags
        bsf INTCON,GIE                 ; enable IRs
; </editor-fold>
 
; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
; **  Main Loop                                         **
; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
loop
       clrwdt                         ; Watchdog-Timer loeschen

        ; *** check for Timer0 (65.5 ms) ***
        btfsc INTCON,T0IF              ; check IR-Flag Timer 0
         call timer0_irf               ; .. if IR-FLAG is set then jump
                    
; <editor-fold  defaultstate="collapsed" desc="DSD">
;++++++++++++++ nur DSD ++++++++++++++
#ifdef dsd2010_grube
        call read_input_dsd            ; Einlesen der R_DATA_IN_x

#ifdef sende_testdaten
        movlw b'10000001' ; Status Flags (#7 = Haus Links/Rechts)
        movwf R_DATA_IN_1 ; Status Flags
        movlw b'00000001' ; Ist Position
        movwf R_DATA_IN_2 ; Ist Position
        movlw b'00000000' ; Soll Position (nicht verwendet)
        movwf R_DATA_IN_3 ; Soll Position (nicht verwendet)
        movlw b'00110001' ; Error Code
        movwf R_DATA_IN_4 ; Error Code

        ; -> Aufruf der entsprechenden MAPP-Routine
        btfsc F_MODE_01
         call mapp_mode1
        btfsc F_MODE_02
         call mapp_mode2
        btfsc F_MODE_03
         call mapp_mode3
        btfsc F_MODE_04
         call mapp_mode5
#endif ;sende_testdaten
#endif
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="AS88">
;++++++++++++++ nur AS ++++++++++++++
#ifdef as88
        ; *** Freeze Abfrage ***
        btfsc IO_DIP1                  ; HIGH heisst DIP nicht aktiv, also off -> kein FREEZE
         goto as_no_freeze             ; .. wenn DIP1 = OFF -> IO_DIP1 = HIGH
        btfss IO_FREEZE                ; HIGH heisst Gleisspanung liegt an -> kein Freeze
         goto loop                     ; .. @ LOW = Optokoppler nicht geschaltet = keine Gleisspannung -> LOOP

as_no_freeze
        call read_input_as88           ; as88
#endif

;++++++++++++++ nur AS 2V6 ++++++++++
#ifdef as88_2v6
        ; *** Freeze Abfrage ***
        btfsc IO_DIP1                  ; HIGH heisst DIP nicht aktiv, also off -> kein FREEZE
         goto as_no_freeze             ; .. wenn DIP1 = OFF -> IO_DIP1 = HIGH
        btfsc IO_FREEZE                ; HIGH heisst Gleisspanung liegt an -> kein Freeze
         goto as_freeze                ; .. @ LOW = Optokoppler nicht geschaltet = keine Gleisspannung -> as_freeze

as_no_freeze
        call read_input_as88           ; as88
as_freeze
#endif; as88_2v6
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="MS88">
;++++++++++++++ nur ms88 ++++++++++++++
#ifdef ms88
        call read_input_ms88           ; ms88
#ifdef sende_testdaten
        movlw b'01101101'
        movwf R_S88DATA_OUT_0
#endif ;sende_testdaten
#endif
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="Weich88">
;++++++++++++++ nur weich88 ++++++++++++++
#ifdef weich88
        call read_input_weich88        ; weich88
#ifdef sende_testdaten
        movlw b'01101101'
        movwf R_S88DATA_OUT_0
#endif ;sende_testdaten
#endif
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="GLS88">
;++++++++++++++ nur GLS ++++++++++++++
#ifdef gls88
        call read_input_gls           ; 
#endif
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="Slow Down">
;++++++++++++++ nur AS 2V6 - 150 ms ++++++++++
#ifdef as88_2v6
; Counter fr langsame Ablufe
       decfsz R_TIMECOUNT_1,F
        goto loop
       decfsz R_TIMECOUNT_2,F
        goto loop
       movlw d'5'
       movwf R_TIMECOUNT_2;
#endif; as88_2v6

;++++++++++++++ nur AS 2V6 - 150 ms ++++++++++
#ifdef as88_2v6
        ; *** 4094 bedienen ***
        btfsc F_4094_MODE_KNIGHT
         call knightrider 
        call update_4094               ; Update der 4094 alle 150 ms
#endif; as88_2v6
; </editor-fold>

        goto loop               

; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
; **  SUB - Routines                                    **
; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
; <editor-fold  defaultstate="collapsed" desc="DEBOUNCE Stream ">
;*********************************
;** Routine: debounce_stream    **
;*********************************
; debounce function: S88 Daten werden nur dann aktualisiert, wenn diese mind. 2x empfangen wurden
; wenn R_S88DATA_PRE_A_x == R_S88DATA_PRE_B_x: R_S88DATA_PRE_B_x -> R_S88DATA_OUT_x wird geladen,
; R_S88DATA_PRE_A_x -> R_S88DATA_PRE_B_x

debounce_stream
#ifdef dsd2010_grube  ; nur DSD hat mehr als 1 Byte
; ++ keine DEBOUNCE Funktion fr DSD -> Daten direkt auf den S88 Bus
        movf  R_S88DATA_PRE_A_0,W ; DSD
        movwf R_S88DATA_OUT_0
        movf  R_S88DATA_PRE_A_1,W ; DSD
        movwf R_S88DATA_OUT_1
        movf  R_S88DATA_PRE_A_2,W ; DSD
        movwf R_S88DATA_OUT_2
        movf  R_S88DATA_PRE_A_3,W ; DSD
        movwf R_S88DATA_OUT_3
        movf  R_S88DATA_PRE_A_4,W ; DSD
        movwf R_S88DATA_OUT_4
        movf  R_S88DATA_PRE_A_5,W ; DSD
        movwf R_S88DATA_OUT_5
        movf  R_S88DATA_PRE_A_6,W ; DSD
        movwf R_S88DATA_OUT_6
        movf  R_S88DATA_PRE_A_7,W ; DSD
        movwf R_S88DATA_OUT_7
        return
#endif ; dsd2010_grube

; ++ CHECK, ob  PRE_A identisch mit PRE_B. Wenn nein -> skip_update
        movf  R_S88DATA_PRE_A_0,W   ; check if _A and _B are same
        subwf R_S88DATA_PRE_B_0,W   ; check if _A and _B are same
        btfss STATUS,Z
         goto skip_update           ; else skip and wait with update of S88 Output Data for next read to get data stable

; ++ PRE_B -> OUT (bernahme der Daten auf den S88 Bus)
#ifdef weich88
        movf  R_S88DATA_PRE_B_0,W   ; W88: Daten in R_S88DATA_OUT_0 bernehmen 
        movwf R_S88DATA_OUT_0
#endif

#ifdef as88
        movf  R_S88DATA_PRE_B_0,W   ; AS88: Daten in R_S88DATA_OUT_0 bernehmen 
        btfss IO_DIP2               ; DIP2 ON -> IO_DIP2 = LOW
         comf R_S88DATA_PRE_B_0,W   ; Bits invertieren
        movwf R_S88DATA_OUT_0
#endif

#ifdef as88_2v6
        movf  R_S88DATA_PRE_B_0,W      ; AS88 2v6: Daten in R_S88DATA_OUT_0 bernehmen 
        movwf R_S88DATA_OUT_0
        movwf R_4094_VALUES_1          ; Daten in R_4094_VALUES bernehmen
        bsf F_4094_UPDATE              ; AS88 2v6: Daten an 4094 bertragen
#endif ;as88_2v6

#ifdef ms88
        movf  R_S88DATA_PRE_B_0,W   ; MS88: Daten in R_S88DATA_OUT_0 bernehmen 
        btfss IO_DIP2               ; DIP2 ON -> IO_DIP2 = LOW
         comf R_S88DATA_PRE_B_0,W   ; Bits invertieren
        movwf R_S88DATA_OUT_0
#endif

skip_update
; ++ PRE_A -> PRE_B (bernahme der Daten ins Vergleichs-Register B)
        movf  R_S88DATA_PRE_A_0,W      ; bernahme R_S88DATA_PRE_A_0 in R_S88DATA_PRE_B_0
        movwf R_S88DATA_PRE_B_0        ; bernahme R_S88DATA_PRE_A_0 in R_S88DATA_PRE_B_0
        return
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="TOIF Flag">
;*********************************
;** Routine:timer0_irf          **
;*********************************
; Aufruf, wenn T0IF gesetzt ist (aus LOOP)
; -> nur DSD2010: DIP einlesen
; -> wenn S88 nicht anliegt -> LED Blinken
timer0_irf
        bcf INTCON,T0IF                ; clear IR-Flag

        decfsz R_TMR0_CNT,F
         return
        movlw d'4'
        movwf R_TMR0_CNT

#ifdef dsd2010_grube
        call read_input_dip            ; DSD: DIP einlesen
#endif ; dsd2010_grube

        movf R_BUSERKENNUNG,F          ; wenn R_BUSERKENNUNG = 0
        btfsc STATUS,Z
         goto led_blink                ; .. dann Blinken

        ; *** S88 liegt an ***         
#ifndef test
#ifndef no_led_status
        bcf IO_LED_STATUS              ; LED continuous ON
#endif ; no_led_status
#endif ; test
        decf R_BUSERKENNUNG,F         
        return

led_blink              
        ; *** S88 liegt NICHT an ***         
        comf PORTB,W

#ifdef dsd2010_grube
        andlw B'01000000'              ; IO_LED_STATUS flashing PORTB,6 dsd2010_grube
        movwf PORTB
#endif

#ifdef as88
        andlw B'10000000'              ; IO_LED_STATUS flashing PORTB,7 as88
        movwf PORTB
#endif

#ifdef as88_2v6
        btfsc F_4094_MODE_KNIGHT
         goto skip_knightrider_init
        ; default-Wert fr 4094 KnightRider-Effekt
        ;movlw d'1'
        ;movwf R_4094_VALUES_1
       ; bsf F_4094_MODE_KNIGHT
skip_knightrider_init
#endif; as88_2v6

#ifdef weich88
 #ifndef test
        andlw B'10000000'              ; IO_LED_STATUS flashing PORTB,7 weich88
        movwf PORTB
 #endif ; test
#endif

#ifdef ms88
 #ifndef test
        andlw B'10000000'              ; IO_LED_STATUS flashing PORTB,7 ms88
        movwf PORTB
 #endif ; test
#endif
        return
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="Read DSD">
;**********************************************
;** Routine: read_input_dsd                  **
;** - Empfang ber die RM Schnittstelle      **
;**********************************************
#ifdef dsd2010_grube
read_input_dsd

        ; Warte auf fallende CLK Flanke
        bcf STATUS,C
        btfsc IO_RM_CLK
         bsf STATUS,C
        rlf R_DATA_IN_CLK,F
        bcf R_DATA_IN_CLK,4   ; nur #0..#3 betrachten
        movlw b'00001100'
        subwf R_DATA_IN_CLK,W
        btfss STATUS,Z
         return 
        clrf R_DATA_IN_CLK

        ; fallende Flanke in CLK gefunden
        bcf F_RM_DATA
        btfsc IO_RM_DATA  ; Latch DATA IN
         bsf F_RM_DATA    ; gelatched, da ggf. Daten schon wieder offline sind

        ; check Strobe
        btfss IO_RM_STROBE
         goto read_input_dsd_01

        ; wenn STROBE = active
        ; check, ob Anzahl Bits OK ist
        movf R_DATA_COUNTER,W
        btfss STATUS,Z
         goto mapp_skip

        ; -> Aufruf der entsprechenden MAPP-Routine -> Daten bernehmen
        btfsc F_MODE_01
         call mapp_mode1
        btfsc F_MODE_02
         call mapp_mode2
        btfsc F_MODE_03
         call mapp_mode3
        btfsc F_MODE_04
#ifdef dsd2010_grube_use_mode_5
         call mapp_mode5
#else
         call mapp_mode4
#endif ; dsd2010_grube_use_mode_5

        movf R_DATA_IN_1,W
        movwf R_DATA_IN_1_OLD

mapp_skip
        ; Restart Zyklus
        movlw d'32'                   ; Counter wieder laden
        movwf R_DATA_COUNTER

read_input_dsd_01
        ; Daten einlesen und durchschieben
        decf R_DATA_COUNTER,F        ; Counter zhlt die Anzahl der empfangenen Bits

        bcf STATUS,C
        btfsc F_RM_DATA              ; gelatched, da ggf. Daten schon wieder offline sind
         bsf STATUS,C
        rrf R_DATA_IN_4,F
        rrf R_DATA_IN_3,F
        rrf R_DATA_IN_2,F
        rrf R_DATA_IN_1,F

        return

read_input_save_clk_h
        bsf F_RM_CLK_OLD 
        return

#endif ; #ifdef dsd2010_grube
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="Read DSD - Read DIP & Switch Mode">
;**********************************************
;** Routine: read_input_dip                  **
;** - DIP einlesen                           **
;**********************************************
#ifdef dsd2010_grube
;    DIP2   DIP1   R_MODE
;     0      0     00000001    ; Mode 1
;     0      1     00000010    ; Mode 2
;     1      0     00000100    ; Mode 3
;     1      1     00001000    ; Mode 4
read_input_dip
        ; Grundstellung: Anhngig von DIP1
        ;    DIP2   DIP1   R_MODE
        ;     x      0     00000001
        ;     x      1     00000100
        movlw b'00000001'
        btfss IO_DIP1                  ; 1 heisst DIP nicht aktiv, also off
         movlw b'00000100'
        movwf R_MODE

        ; wenn DIP2, dann um 1 nach links schieben
        ;    DIP2   DIP1   R_MODE
        ;     0      0     00000001
        ;     1      0     00000010
        ;     0      1     00000100
        ;     1      1     00001000
        bcf STATUS,C                   ; clear C for RLF
        btfss IO_DIP2                  ; 1 heisst DIP nicht aktiv, also off
         rlf R_MODE,F

        return
#endif ; #ifdef dsd2010_grube

;**********************************************
;** Routine: mapp_mode1                      **
;** - Mode1 (DIP) Ausgangsregister Laden     **
;**********************************************
#ifdef dsd2010_grube
mapp_mode1
        ; S88 Output: Flags + Position (BCD)
        movlw grube_mode1_anz
        movwf R_S88BIT_ANZAHL

        movf R_DATA_IN_1,W             ; Flags
        movwf R_S88DATA_PRE_A_0        ; DSD
        incf R_DATA_IN_2,W             ; Istposition
        movwf R_S88DATA_PRE_A_1        ; DSD

        call debounce_stream           ; check bernahme in S88 Output Date (Debounce DSD Mode 1)
        return
#endif ; #ifdef dsd2010_grube

;**********************************************
;** Routine: mapp_mode2                      **
;** - Mode1 (DIP) Ausgangsregister Laden     **
;**********************************************
#ifdef dsd2010_grube
mapp_mode2
        ; S88 Output: Flags + Position (BCD) + Error
        movlw grube_mode2_anz
        movwf R_S88BIT_ANZAHL

        movf R_DATA_IN_1,W             ; Flags
        movwf R_S88DATA_PRE_A_0        ; DSD
        incf R_DATA_IN_2,W             ; Istposition
        movwf R_S88DATA_PRE_A_1        ; DSD
        movf R_DATA_IN_4,W             ; Errorcode in Reg. 2
        movwf R_S88DATA_PRE_A_2        ; DSD
 
        call debounce_stream           ; check bernahme in S88 Output Date (Debounce DSD Mode 2)
        return
#endif ; #ifdef dsd2010_grube

;**********************************************
;** Routine: mapp_mode3                      **
;** - Mode1 (DIP) Ausgangsregister Laden     **
;**********************************************
#ifdef dsd2010_grube
mapp_mode3
        movlw grube_mode3_anz
        movwf R_S88BIT_ANZAHL
 
        movf R_DATA_IN_1,W             ; Flags
        movwf R_S88DATA_PRE_A_0        ; DSD

        ; Position umwanden in Bit-Orientiert
        incf R_DATA_IN_2,W             ; Position BCD
        movwf R_TEMP                   ; --> Counter

        clrf R_DATA_TEMP1
        clrf R_DATA_TEMP2
        clrf R_DATA_TEMP3

        bsf STATUS,C

mapp_mode3_loop
        rlf R_DATA_TEMP1,F             ;-> R_S88DATA_PRE_A_1,F
        rlf R_DATA_TEMP2,F             ;-> R_S88DATA_PRE_A_2,F
        rlf R_DATA_TEMP3,F             ;-> R_S88DATA_PRE_A_3,F
        bcf STATUS,C
        decfsz R_TEMP,F
         goto mapp_mode3_loop

        movf R_DATA_TEMP1,W
        movwf R_S88DATA_PRE_A_1
        movf R_DATA_TEMP2,W
        movwf R_S88DATA_PRE_A_2
        movf R_DATA_TEMP3,W
        movwf R_S88DATA_PRE_A_3

        call debounce_stream           ; check bernahme in S88 Output Date  (Debounce DSD Mode 3)
        return
#endif ; #ifdef dsd2010_grube

;**********************************************
;** Routine: mapp_mode4                      **
;** - Mode1 (DIP) Ausgangsregister Laden     **
;**********************************************
#ifdef dsd2010_grube
#ifndef dsd2010_grube_use_mode_5
mapp_mode4
        movlw grube_mode4_anz
        movwf R_S88BIT_ANZAHL
 
        movf R_DATA_IN_1,W             ; Flags
        movwf R_S88DATA_PRE_A_0        ; DSD

        ; Position umwanden in Bit-Orientiert
        incf R_DATA_IN_2,W             ; Position BCD
        movwf R_TEMP                   ; --> Counter

        clrf R_DATA_TEMP1
        clrf R_DATA_TEMP2
        clrf R_DATA_TEMP3

        bsf STATUS,C

mapp_mode4_loop
        rlf R_DATA_TEMP1,F             ;-> R_S88DATA_PRE_A_1,F
        rlf R_DATA_TEMP2,F             ;-> R_S88DATA_PRE_A_2,F
        rlf R_DATA_TEMP3,F             ;-> R_S88DATA_PRE_A_3,F
        bcf STATUS,C
        decfsz R_TEMP,F
         goto mapp_mode4_loop

        movf R_DATA_TEMP1,W
        movwf R_S88DATA_PRE_A_1
        movf R_DATA_TEMP2,W
        movwf R_S88DATA_PRE_A_2
        movf R_DATA_TEMP3,W
        movwf R_S88DATA_PRE_A_3

        movf R_DATA_IN_4,W             ; Errorcode in Reg. 4
        movwf R_S88DATA_PRE_A_4

        call debounce_stream           ; check bernahme in S88 Output Date  (Debounce DSD Mode 4)
        return
#endif ; dsd2010_grube_use_mode_5
#endif ; #ifdef dsd2010_grube

;**********************************************
;** Routine: mapp_mode5                      **
;** - Mode5 Ausgangsregister Laden           **
;**********************************************
#ifdef dsd2010_grube
#ifdef dsd2010_grube_use_mode_5
mapp_mode5
#ifdef test
 call toggle_test_1
#endif ; test

        btfss F_SEC_HALF_OLD
         goto f_sec_half_old_false
f_sec_half_old_true
        btfss F_SEC_HALF
         return ; F_SEC_HALF = FALSE & F_SEC_HALF_OLD = TRUE
        goto check_f_sec_half_done

f_sec_half_old_false
        btfsc F_SEC_HALF
         return ; F_SEC_HALF = TRUE & F_SEC_HALF_OLD = FALSE

check_f_sec_half_done

        movlw grube_mode5_anz
        movwf R_S88BIT_ANZAHL
 
        movf R_DATA_IN_1,W             ; Flags
        movwf R_S88DATA_PRE_A_0        ; DSD

        ; Position umwanden in Bit-Orientiert
        incf R_DATA_IN_2,W             ; Position BCD
        movwf R_TEMP                   ; --> Counter

        clrf R_DATA_TEMP1
        clrf R_DATA_TEMP2
        clrf R_DATA_TEMP3

        bsf STATUS,C

mapp_mode5_loop
#ifdef test
 call toggle_test_2
#endif ; test
        rlf R_DATA_TEMP1,F             ;-> R_S88DATA_PRE_A_1,F
        rlf R_DATA_TEMP2,F             ;-> R_S88DATA_PRE_A_2,F
        rlf R_DATA_TEMP3,F             ;-> R_S88DATA_PRE_A_3,F
        bcf STATUS,C
        decfsz R_TEMP,F
         goto mapp_mode5_loop

        movf R_DATA_IN_4,W             ; Errorcode in Reg. 7
        movwf R_S88DATA_PRE_A_7

        btfss R_DATA_IN_1,7            ; wenn "erste Hlfte"
         goto mapp_mode5_first_half    ; .. mapp_mode5_first_half

mapp_mode5_second_half                 ; Haus = links -> Umladen in die "Hohen Register"
#ifdef test
 call toggle_test_3
#endif ; test
        clrf R_S88DATA_PRE_A_1
        clrf R_S88DATA_PRE_A_2
        clrf R_S88DATA_PRE_A_3

        movf R_DATA_TEMP1,W
        movwf R_S88DATA_PRE_A_4
        movf R_DATA_TEMP2,W
        movwf R_S88DATA_PRE_A_5
        movf R_DATA_TEMP3,W
        movwf R_S88DATA_PRE_A_6

        call debounce_stream           ; check bernahme in S88 Output Date (Debounce DSD Mode 5)
        return

mapp_mode5_first_half                  ; Haus = links -> Umladen in die "Unteren Register"
        clrf R_S88DATA_PRE_A_4
        clrf R_S88DATA_PRE_A_5
        clrf R_S88DATA_PRE_A_6

        movf R_DATA_TEMP1,W
        movwf R_S88DATA_PRE_A_1
        movf R_DATA_TEMP2,W
        movwf R_S88DATA_PRE_A_2
        movf R_DATA_TEMP3,W
        movwf R_S88DATA_PRE_A_3

        call debounce_stream           ; check bernahme in S88 Output Date (Debounce DSD Mode 5)
        return

#endif ; dsd2010_grube_use_mode_5
#endif ; #ifdef dsd2010_grube
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="Weich88">
;**********************************************
;** Routine: read_input_weich88              **
;** - Ausgangsregister Laden                 **
;**********************************************
#ifdef weich88
read_input_weich88
#ifdef sende_testdaten ; keine nderung der Ausgangsdaten vornehmen
        return
#endif ; sende_testdaten


; **** Load Ports into Registers (latch inputs) ****
        movf PORTB,W
        movwf R_PORTB
        movf PORTC,W
        movwf R_PORTC

; **** Debounce X1 - Bit #0 ****
check_w88_1 btfsc IO_RM1_A             ; Port high -> no train -> Bit clear
        goto check_w88_1h
check_w88_1l;*** PORT is low ***
        movlw CNT_DEBOUNCE
        addwf R_DEB1,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_w88_1y              ; if R_DEB1 > 255 then jump
        goto check_w88_2                ; else next
check_w88_1h;*** PORT is high ***
        movlw CNT_DEBOUNCE
        subwf R_DEB1,F
        btfss STATUS,C                 ; check for CARRY
         goto check_w88_1n              ; if R_DEB1 < 0 then jump
        goto check_w88_2                ; else next
check_w88_1y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB1                   ; .. to FF
        bsf R_S88DATA_PRE_A_0,0        ; W88
        goto check_w88_2                ; then  next
check_w88_1n;*** INPUT is not active *** 
        clrf R_DEB1                    ; clear counter
        bcf R_S88DATA_PRE_A_0,0        ; W88
        goto check_w88_2               ; then  next

; **** Debounce X2 - Bit #1 ****
check_w88_2 btfsc IO_RM1_B             ; Port high -> no train -> Bit clear
        goto check_w88_2h
check_w88_2l;*** PORT is low ***
        movlw CNT_DEBOUNCE
        addwf R_DEB2,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_w88_2y              ; if R_DEB2 > 255 then jump
        goto check_w88_3                ; else next
check_w88_2h;*** PORT is high ***
        movlw CNT_DEBOUNCE
        subwf R_DEB2,F
        btfss STATUS,C                 ; check for CARRY
         goto check_w88_2n              ; if R_DEB2 < 0 then jump
        goto check_w88_3                ; else next
check_w88_2y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB2                   ; .. to FF
        bsf R_S88DATA_PRE_A_0,1        ; W88
        goto check_w88_3                ; then  next
check_w88_2n;*** INPUT is not active *** 
        clrf R_DEB2                    ; clear counter
        bcf R_S88DATA_PRE_A_0,1        ; W88
        goto check_w88_3                ; then  next

; **** Debounce X3 - Bit #2 ****
check_w88_3 btfsc IO_RM2_A             ; Port high -> no train -> Bit clear
        goto check_w88_3h
check_w88_3l;*** PORT is low ***
        movlw CNT_DEBOUNCE
        addwf R_DEB3,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_w88_3y              ; if R_DEB3 > 255 then jump
        goto check_w88_4                ; else next
check_w88_3h;*** PORT is high ***
        movlw CNT_DEBOUNCE
        subwf R_DEB3,F
        btfss STATUS,C                 ; check for CARRY
         goto check_w88_3n              ; if R_DEB3 < 0 then jump
        goto check_w88_4                ; else next
check_w88_3y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB3                   ; .. to FF
        bsf R_S88DATA_PRE_A_0,2        ; W88
        goto check_w88_4                ; then  next
check_w88_3n;*** INPUT is not active *** 
        clrf R_DEB3                    ; clear counter
        bcf R_S88DATA_PRE_A_0,2        ; W88
        goto check_w88_4                ; then  next


; **** Debounce X4 - Bit #3 ****
check_w88_4 btfsc IO_RM2_B             ; Port high -> no train -> Bit clear
        goto check_w88_4h
check_w88_4l;*** PORT is low ***
        movlw CNT_DEBOUNCE
        addwf R_DEB4,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_w88_4y              ; if R_DEB4 > 255 then jump
        goto check_w88_5                ; else next
check_w88_4h;*** PORT is high ***
        movlw CNT_DEBOUNCE
        subwf R_DEB4,F
        btfss STATUS,C                 ; check for CARRY
         goto check_w88_4n              ; if R_DEB4 < 0 then jump
        goto check_w88_5                ; else next
check_w88_4y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB4                   ; .. to FF
        bsf R_S88DATA_PRE_A_0,3        ; W88
        goto check_w88_5                ; then  next
check_w88_4n;*** INPUT is not active *** 
        clrf R_DEB4                    ; clear counter
        bcf R_S88DATA_PRE_A_0,3        ; W88
        goto check_w88_5                ; then  next

; **** Debounce X5 - Bit #4 ****
check_w88_5 btfsc IO_RM3_A             ; Port high -> no train -> Bit clear
        goto check_w88_5h
check_w88_5l;*** PORT is low ***
        movlw CNT_DEBOUNCE
        addwf R_DEB5,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_w88_5y              ; if R_DEB5 > 255 then jump
        goto check_w88_6                ; else next
check_w88_5h;*** PORT is high ***
        movlw CNT_DEBOUNCE
        subwf R_DEB5,F
        btfss STATUS,C                 ; check for CARRY
         goto check_w88_5n              ; if R_DEB5 < 0 then jump
        goto check_w88_6                ; else next
check_w88_5y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB5                   ; .. to FF
        bsf R_S88DATA_PRE_A_0,4        ; W88
        goto check_w88_6                ; then  next
check_w88_5n;*** INPUT is not active *** 
        clrf R_DEB5                    ; clear counter
        bcf R_S88DATA_PRE_A_0,4        ; W88
        goto check_w88_6                ; then  next

; **** Debounce X6 - Bit #5 ****
check_w88_6 btfsc IO_RM3_B             ; Port high -> no train -> Bit clear
        goto check_w88_6h
check_w88_6l;*** PORT is low ***
        movlw CNT_DEBOUNCE
        addwf R_DEB6,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_w88_6y              ; if R_DEB6 > 255 then jump
        goto check_w88_7                ; else next
check_w88_6h;*** PORT is high ***
        movlw CNT_DEBOUNCE
        subwf R_DEB6,F
        btfss STATUS,C                 ; check for CARRY
         goto check_w88_6n              ; if R_DEB6 < 0 then jump
        goto check_w88_7                ; else next
check_w88_6y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB6                   ; .. to FF
        bsf R_S88DATA_PRE_A_0,5        ; W88
        goto check_w88_7                ; then  next
check_w88_6n;*** INPUT is not active *** 
        clrf R_DEB6                    ; clear counter
        bcf R_S88DATA_PRE_A_0,5        ; W88
        goto check_w88_7                ; then  next

; **** Debounce X7 - Bit #6 ****
check_w88_7 btfsc IO_RM4_A             ; Port high -> no train -> Bit clear
        goto check_w88_7h
check_w88_7l;*** PORT is low ***
        movlw CNT_DEBOUNCE
        addwf R_DEB7,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_w88_7y              ; if R_DEB7 > 255 then jump
        goto check_w88_8                ; else next
check_w88_7h;*** PORT is high ***
        movlw CNT_DEBOUNCE
        subwf R_DEB7,F
        btfss STATUS,C                 ; check for CARRY
         goto check_w88_7n              ; if R_DEB7 < 0 then jump
        goto check_w88_8                ; else next
check_w88_7y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB7                   ; .. to FF
        bsf R_S88DATA_PRE_A_0,6        ; W88
        goto check_w88_8                ; then  next
check_w88_7n;*** INPUT is not active *** 
        clrf R_DEB7                    ; clear counter
        bcf R_S88DATA_PRE_A_0,6        ; W88
        goto check_w88_8                ; then  next

; **** Debounce X8 - Bit #7 ****
check_w88_8 btfsc IO_RM4_B             ; Port high -> no train -> Bit clear
        goto check_w88_8h
check_w88_8l;*** PORT is low ***
        movlw CNT_DEBOUNCE
        addwf R_DEB8,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_w88_8y              ; if R_DEB8 > 255 then jump
        goto check_w88_9                ; else next
check_w88_8h;*** PORT is high ***
        movlw CNT_DEBOUNCE
        subwf R_DEB8,F
        btfss STATUS,C                 ; check for CARRY
         goto check_w88_8n              ; if R_DEB8 < 0 then jump
        goto check_w88_9                ; else next
check_w88_8y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB8                   ; .. to FF
        bsf R_S88DATA_PRE_A_0,7        ; W88
        goto check_w88_9                ; then  next
check_w88_8n;*** INPUT is not active *** 
        clrf R_DEB8                    ; clear counter
        bcf R_S88DATA_PRE_A_0,7        ; W88
        goto check_w88_9                ; then  next

check_w88_9 
        call debounce_stream           ; check bernahme in S88 Output Date (Debounce W88)
        return

#endif ;weich88
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="MS88">
;**********************************************
;** Routine: read_input_ms88                 **
;** - Ausgangsregister Laden                 **
;** Aufruf ca. alle 500s (CS3, Bitzeit 100us)**
;** Aufruf ca. alle 130s (kein S88-CLK)     **
;**********************************************
#ifdef ms88
read_input_ms88
; **** Load Ports into Registers (latch inputs) ****
        movf PORTB,W
        movwf R_PORTB
        movf PORTC,W
        movwf R_PORTC

        ; Map Inputs aus den Ports in Register R_INPUT
        clrf R_INPUTS
        btfss IO_RM1
         bsf R_INPUTS,0
        btfss IO_RM2
         bsf R_INPUTS,1
        btfss IO_RM3
         bsf R_INPUTS,2
        btfss IO_RM4
         bsf R_INPUTS,3
        btfss IO_RM5
         bsf R_INPUTS,4
        btfss IO_RM6
         bsf R_INPUTS,5
        btfss IO_RM7
         bsf R_INPUTS,6
        btfss IO_RM8
         bsf R_INPUTS,7

        movlw b'00000001'
        movwf R_INPUTLOOP_FLAG         ; fr FLAG Maskierung etc.
        clrf R_INPUTLOOP_CNT           ; fr indirekte Adressierung      

; **** Eingnge Abfragen ****
check_ms88_loop 
        ; Check FREEZE (S88 Impulsverlngerung)
        movf R_FLAGS_FREEZE,W
        andwf R_INPUTLOOP_FLAG,W
        btfss STATUS,Z                 ; wenn !=ZERO, dann kein GOTO
         goto check_ms88_next          ; wenn FREEZE aktiv (HIGH), dann springe

        ; RAM Adresse DEBOUNCE laden
        movlw RAM_R_DEB
        addwf R_INPUTLOOP_CNT,W
        movwf FSR

        ; Check, ob der Input HIGH oder LOW ist
        movf R_INPUTS,W                ; gemappte INPUTs
        andwf R_INPUTLOOP_FLAG,W       ; .. maskieren
        btfsc STATUS,Z                 ; wenn ZERO, dann kein GOTO
         goto check_ms88_input_idle    ; wenn INPUT nicht aktiv (LOW), dann springe

;*** PORT is low = ACTIVE ***
check_ms88_input_activ 
        ; Wert ndern
        movlw CNT_DEBOUNCE_ADD
        addwf INDF,F                   ; DEBOUNCE-Wert vergrern
        btfsc STATUS,C                 ; check for CARRY
         goto check_ms88_aktiv         ; if R_DEB1 > 255 then jump -> AKTIV
        goto check_ms88_next              ; else next

;*** PORT is high = NOT ACTIVE ***
check_ms88_input_idle 
        movlw CNT_DEBOUNCE_SUB
        subwf INDF,F                   ; DEBOUNCE-Wert verkleinern
        btfss STATUS,C                 ; check for CARRY
         goto check_ms88_idle          ; if R_DEB1 < 0 then jump -> IDLE
        goto check_ms88_next           ; else next

;*** INPUT is active ***
check_ms88_aktiv
        movlw h'ff'                    ; clear counter
        movwf INDF                     ; .. to FF

        ; BIT auf AKTIV setzen
        movf R_INPUTLOOP_FLAG,W
        iorwf R_S88DATA_PRE_A_0,F      ; MS: Bit auf AKTIV setzen im PRE Register (fr Debounce)

        ; Ausschalt-Verzgerung: bitspezifischen Counter laden
        ; RAM Adresse PS_COUNT laden
        movlw RAM_R_PS_COUNT
        addwf R_INPUTLOOP_CNT,W
        movwf FSR
        ; Wert ndern
        movlw CNT_PS_FREEZE_1          ; Mindest-Anzahl an S88 Zyklen, fr die das Bit aktiv bleiben soll 
        btfss IO_DIP1                  ; DATEN Erkennung Langsam @ DIP1 = ON = LOW -> SKIP @ HIGH
         movlw CNT_PS_FREEZE_2
        movwf INDF

        ; FREEZE aktivieren, wenn dieses BIT gerade aktiv wird (also vorher noch nicht aktiv war)
        movf R_S88DATA_OUT_0,W         ; wenn Daten-Bit schon erkannt wurde (also Aktiv ist)
        andwf R_INPUTLOOP_FLAG,W
        btfss STATUS,Z                 ; wenn !=ZERO, dann kein GOTO
         goto check_ms88_next          ; wenn Flag in R_S88DATA_OUT_0 nicht aktiv (LOW), dann springe

        movf R_INPUTLOOP_FLAG,W
        iorwf R_FLAGS_FREEZE,F         ; Flag FREEZE wird gesetzt
        goto check_ms88_next           ; then next

;*** INPUT is NOT active ***
check_ms88_idle
        clrf INDF                      ; clear DEBOUNCE-Wert counter

        ; BIT auf IDLE setzen
        comf R_INPUTLOOP_FLAG,W        ; invertieren und in W
        andwf R_S88DATA_PRE_A_0,F      ; MS: Bit lschen (Idle) im PRE Register (fr Debounce)

;*** nchster Eingang? ***
check_ms88_next
        incf R_INPUTLOOP_CNT,F         ; fr indirekte Adressierung      
        bcf STATUS,C
        rlf R_INPUTLOOP_FLAG,F         ; fr FLAG Masierung etc.
        btfss STATUS,C
         goto check_ms88_loop

;*** LOOP beendet, alle Eingnge abgefragt ***
        call debounce_stream           ; check bernahme in S88 Output Date (Debounce MS88)

        ; S88 PS Impuls aufgetreten?
; bsf F_PS_OK
        btfss F_PS_OK
         return

        bcf F_PS_OK

        decf R_PS_COUNT_1,F
        btfsc STATUS,Z
         bcf FREEZE_OUT1
        decf R_PS_COUNT_2,F
        btfsc STATUS,Z
         bcf FREEZE_OUT2
        decf R_PS_COUNT_3,F
        btfsc STATUS,Z
         bcf FREEZE_OUT3
        decf R_PS_COUNT_4,F
        btfsc STATUS,Z
         bcf FREEZE_OUT4
        decf R_PS_COUNT_5,F
        btfsc STATUS,Z
         bcf FREEZE_OUT5
        decf R_PS_COUNT_6,F
        btfsc STATUS,Z
         bcf FREEZE_OUT6
        decf R_PS_COUNT_7,F
        btfsc STATUS,Z
         bcf FREEZE_OUT7
        decf R_PS_COUNT_8,F
        btfsc STATUS,Z
         bcf FREEZE_OUT8

#ifdef test
; call toggle_test_3
#endif ; test        


        return

#endif ;ms88
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="AS88">
;**********************************************
;** Routine: read_input_as88              **
;** - Ausgangsregister Laden                 **
;**********************************************
#ifdef as88
read_input_as88
#ifdef sende_testdaten ; keine nderung der Ausgangsdaten vornehmen
        return
#endif ; sende_testdaten

; **** Load Ports into Registers (latch inputs) ****
        bsf IO_POWER_LED                ; LED Anzeige AUS (durch BC807 invertierend)
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us

        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us
        goto $+1                        ; Wartezeit 1 us

; **** Load Ports into Registers (latch inputs) ****
        movf PORTB,W
        movwf R_PORTB
        movf PORTC,W
        movwf R_PORTC

        bcf IO_POWER_LED                ; LED Anzeige wieder AN

        ; Map Inputs aus den Ports in Register R_INPUT
        clrf R_INPUTS
        btfss IO_RM1
         bsf R_INPUTS,0
        btfss IO_RM2
         bsf R_INPUTS,1
        btfss IO_RM3
         bsf R_INPUTS,2
        btfss IO_RM4
         bsf R_INPUTS,3
        btfss IO_RM5
         bsf R_INPUTS,4
        btfss IO_RM6
         bsf R_INPUTS,5
        btfss IO_RM7
         bsf R_INPUTS,6
        btfss IO_RM8
         bsf R_INPUTS,7

        movlw b'00000001'
        movwf R_INPUTLOOP_FLAG         ; fr FLAG Masierung etc.
        clrf R_INPUTLOOP_CNT           ; fr indirekte Adressierung      

; **** Eingnge Abfragen ****
check_as88_loop 

        ; RAM Adresse DEBOUNCE laden
        movlw RAM_R_DEB
        addwf R_INPUTLOOP_CNT,W
        movwf FSR

        ; Check, ob der Input HIGH oder LOW ist
        movf R_INPUTS,W
        andwf R_INPUTLOOP_FLAG,W
        btfsc STATUS,Z                 ; wenn ZERO, dann kein GOTO
         goto check_as88_input_idle    ; wenn INPUT nicht aktiv (HIGH), dann springe

;*** PORT is LOW = ACTIVE ***
check_as88_input_activ 
#ifdef test
 call toggle_test_1
#endif ; test        
        ; Wert ndern
        movlw CNT_DEBOUNCE_ADD
        addwf INDF,F                   ; DEBOUNCE-Wert vergrern
        btfsc STATUS,C                 ; check for CARRY
         goto check_as88_aktiv         ; if R_DEB1 > 255 then jump -> AKTIV
        goto check_as88_next              ; else next


;*** PORT is HIGH = NOT ACTIVE ***
check_as88_input_idle 
        movlw CNT_DEBOUNCE_SUB
        subwf INDF,F                   ; DEBOUNCE-Wert verkleinern
        btfss STATUS,C                 ; check for CARRY
         goto check_as88_idle          ; if R_DEB1 < 0 then jump -> IDLE
        goto check_as88_next           ; else next

;*** INPUT is active ***
check_as88_aktiv
        movlw h'ff'                    ; clear counter
        movwf INDF                     ; .. to FF

        ; BIT auf AKTIV setzen
        movf R_INPUTLOOP_FLAG,W
        iorwf R_S88DATA_PRE_A_0,F      ; AS: Bit auf AKTIV setzen im PRE Register (fr Debounce)

        goto check_as88_next           ; then next



;*** INPUT is NOT active ***
check_as88_idle
        clrf INDF                      ; clear DEBOUNCE-Wert counter

        ; BIT auf IDLE setzen
        comf R_INPUTLOOP_FLAG,W        ; invertieren und in W
        andwf R_S88DATA_PRE_A_0,F      ; AS: Bit lschen (Idle) im PRE Register (fr Debounce)



;*** nchster Eingang? ***
check_as88_next
        incf R_INPUTLOOP_CNT,F         ; fr indirekte Adressierung      
        bcf STATUS,C
        rlf R_INPUTLOOP_FLAG,F         ; fr FLAG Masierung etc.
        btfss STATUS,C
         goto check_as88_loop


;*** LOOP beendet, alle Eingnge abgefragt ***
        call debounce_stream           ; check bernahme in S88 Output Date (Debounce AS88)
        return

#endif ;as88
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="AS88 2v6">
;**********************************************
;** Routine: read_input_as88_2v6             **
;** - Ausgangsregister Laden                 **
;**********************************************
#ifdef as88_2v6
read_input_as88
#ifdef sende_testdaten ; keine nderung der Ausgangsdaten vornehmen
        return
#endif ; sende_testdaten

; **** Load Ports into Registers (latch inputs) ****
        movf PORTB,W
        movwf R_PORTB
        movf PORTC,W
        movwf R_PORTC

        ; Map Inputs aus den Ports in Register R_INPUT
        clrf R_INPUTS
        btfss IO_RM1
         bsf R_INPUTS,0
        btfss IO_RM2
         bsf R_INPUTS,1
        btfss IO_RM3
         bsf R_INPUTS,2
        btfss IO_RM4
         bsf R_INPUTS,3
        btfss IO_RM5
         bsf R_INPUTS,4
        btfss IO_RM6
         bsf R_INPUTS,5
        btfss IO_RM7
         bsf R_INPUTS,6
        btfss IO_RM8
         bsf R_INPUTS,7

        movlw b'00000001'
        movwf R_INPUTLOOP_FLAG         ; fr FLAG Masierung etc.
        clrf R_INPUTLOOP_CNT           ; fr indirekte Adressierung      

; **** Eingnge Abfragen ****
check_as88_loop 

        ; RAM Adresse DEBOUNCE laden
        movlw RAM_R_DEB
        addwf R_INPUTLOOP_CNT,W
        movwf FSR

        ; Check, ob der Input HIGH oder LOW ist
        movf R_INPUTS,W
        andwf R_INPUTLOOP_FLAG,W
        btfsc STATUS,Z                 ; wenn ZERO, dann kein GOTO
         goto check_as88_input_idle    ; wenn INPUT nicht aktiv (HIGH), dann springe

;*** PORT is LOW = ACTIVE ***
check_as88_input_activ 
#ifdef test
 call toggle_test_1
#endif ; test        
        ; Wert ndern
        movlw CNT_DEBOUNCE_ADD
        addwf INDF,F                   ; DEBOUNCE-Wert vergrern
        btfsc STATUS,C                 ; check for CARRY
         goto check_as88_aktiv         ; if R_DEB1 > 255 then jump -> AKTIV
        goto check_as88_next              ; else next


;*** PORT is HIGH = NOT ACTIVE ***
check_as88_input_idle 
        movlw CNT_DEBOUNCE_SUB
        subwf INDF,F                   ; DEBOUNCE-Wert verkleinern
        btfss STATUS,C                 ; check for CARRY
         goto check_as88_idle          ; if R_DEB1 < 0 then jump -> IDLE
        goto check_as88_next           ; else next

;*** INPUT is active ***
check_as88_aktiv
        movlw h'ff'                    ; clear counter
        movwf INDF                     ; .. to FF

        ; BIT auf AKTIV setzen
        movf R_INPUTLOOP_FLAG,W
        iorwf R_S88DATA_PRE_A_0,F      ; AS 2v6: Bit auf AKTIV setzen im PRE Register (fr Debounce)

        goto check_as88_next           ; then next



;*** INPUT is NOT active ***
check_as88_idle
        clrf INDF                      ; clear DEBOUNCE-Wert counter

        ; BIT auf IDLE setzen
        comf R_INPUTLOOP_FLAG,W        ; invertieren und in W
        andwf R_S88DATA_PRE_A_0,F      ; AS 2v6: Bit lschen (Idle) im PRE Register (fr Debounce)



;*** nchster Eingang? ***
check_as88_next
        incf R_INPUTLOOP_CNT,F         ; fr indirekte Adressierung      
        bcf STATUS,C
        rlf R_INPUTLOOP_FLAG,F         ; fr FLAG Masierung etc.
        btfss STATUS,C
         goto check_as88_loop


;*** LOOP beendet, alle Eingnge abgefragt ***
        call debounce_stream           ; check bernahme in S88 Output Date (Debounce AS88 2v6)
        return

#endif ;as88_2v6

; ******************************************
; **  update_4094                         **
; ******************************************
#ifdef as88_2v6
update_4094
       btfss F_4094_UPDATE             ; AS88 2v6: Update 4094 ntig?
        return

       ; Wert gleich OLD? Dann nicht nochmal bertragen
       movf R_4094_VALUES_1,W
       subwf R_4094_VALUES_1_OLD,W
       btfsc STATUS,Z                  ; Werte identisch?
        return                         ; .. dann RETURN

       bcf IO_DATA_4094
       btfsc R_4094_VALUES_1,7
        bsf IO_DATA_4094
       ;CLK Impuls
       bsf  IO_CLK_4094
       bcf IO_CLK_4094

       bcf IO_DATA_4094
       btfsc R_4094_VALUES_1,6
        bsf IO_DATA_4094
       ;CLK Impuls
       bsf IO_CLK_4094
       bcf IO_CLK_4094

       bcf IO_DATA_4094
       btfsc R_4094_VALUES_1,5
        bsf IO_DATA_4094
       ;CLK Impuls
       bsf IO_CLK_4094
       bcf IO_CLK_4094

       bcf IO_DATA_4094
       btfsc R_4094_VALUES_1,4
        bsf IO_DATA_4094
       ;CLK Impuls
       bsf IO_CLK_4094
       bcf IO_CLK_4094

       bcf IO_DATA_4094
       btfsc R_4094_VALUES_1,3
        bsf IO_DATA_4094
       ;CLK Impuls
       bsf IO_CLK_4094
       bcf IO_CLK_4094

       bcf IO_DATA_4094
       btfsc R_4094_VALUES_1,2
        bsf IO_DATA_4094
       ;CLK Impuls
       bsf IO_CLK_4094
       bcf IO_CLK_4094

       bcf IO_DATA_4094
       btfsc R_4094_VALUES_1,1
        bsf IO_DATA_4094
       ;CLK Impuls
       bsf IO_CLK_4094
       bcf IO_CLK_4094

       bcf IO_DATA_4094
       btfsc R_4094_VALUES_1,0
        bsf IO_DATA_4094
       ;CLK Impuls
       bsf IO_CLK_4094
       bcf IO_CLK_4094

       ;Strobe Impuls
       bsf IO_STROBE_4094
       bcf IO_STROBE_4094

       ; Flag lschen, dass 4094 Update bentigt
       bcf F_4094_UPDATE               ; AS88 2v6 Daten bertragen fertig

       ; Latch letztes VALUE Register
       movf R_4094_VALUES_1,W
       movwf R_4094_VALUES_1_OLD

       return
       
#endif ;as88_2v6
; ******************************************
; **  knightrider-Effekt                  **
; ******************************************
#ifdef as88_2v6
knightrider
        btfss F_4094_MODE_KNIGHT   ; RETURN, wenn nicht KNIGHT Mode -> dann hier nix machen
         return;
        bsf F_4094_UPDATE          ; AS88 2v6: Knightrider Effekt

        btfss F_4094_KNIGHT_LEFT
         goto knight_r

knight_l  ;Linkssrum
        bcf STATUS,C
        rlf R_4094_VALUES_1,F
        
        ; wurde die 1 herausgeschoben? 
        btfss STATUS,C
         return                        ; Nein, 0 wurde herausgeschoben
        bcf F_4094_KNIGHT_LEFT
        rrf R_4094_VALUES_1,F          ; 1 wieder zurckschieben
        return

knight_r  ;Rechtsrum
        bcf STATUS,C
        rrf R_4094_VALUES_1,F
        
        ; wurde die 1 herausgeschoben? 
        btfss STATUS,C
         return                        ; Nein, 0 wurde haurausgeschoben
        bsf F_4094_KNIGHT_LEFT
        rlf R_4094_VALUES_1,F          ; 1 wieder zurckschieben
        return
#endif ; as88_2v6
; </editor-fold>

; <editor-fold  defaultstate="collapsed" desc="Test-Funktionen">
;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: toggle_xxx         **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef weich88
toggle_test_1  ; test routine
     movf PORTC,W ;test
     xorlw b'10000000' ;test        ; toggle PORTC,7
     movwf PORTC ;test
     return

toggle_test_2  ; test routine
     movf PORTC,W ;test
     xorlw b'00100000' ;test        ; toggle PORTC,5
     movwf PORTC ;test
     return

toggle_test_3  ; test routine
     movf PORTB,W ;test
     xorlw b'10000000' ;test        ; toggle PORTB,7 (LED)
     movwf PORTB ;test
     return
#endif ; weich88

#ifdef dsd2010_grube
toggle_test_1  ; test routine
     movf PORTC,W ;test
     xorlw b'00100000' ;test        ; toggle PORTC,5
     movwf PORTC ;test
     return

toggle_test_2  ; test routine
     movf PORTC,W ;test
     xorlw b'00010000' ;test        ; toggle PORTC,4
     movwf PORTC ;test
     return

toggle_test_3  ; test routine
     movf PORTC,W ;test
     xorlw b'00001000' ;test        ; toggle PORTXC,3
     movwf PORTC ;test
     return
#endif ; dsd2010_grube

#ifdef ms88
toggle_test_1  ; test routine
     movf PORTB,W ;test
     xorlw b'10000000' ;test        ; toggle PORTB,7 (LED)
     movwf PORTB ;test
     return

toggle_test_2  ; test routine
     movf PORTA,W ;test
     xorlw b'00000001' ;test        ; toggle PORTA,0 (P_DATA)
     movwf PORTA ;test
     return

toggle_test_3  ; test routine
     movf PORTA,W ;test
     xorlw b'00000010' ;test        ; toggle PORTA,1 (P_CLK)
     movwf PORTA ;test
     return

#ifdef test_use_DIP_as_OUT
toggle_test_4  ; test routine
     movf PORTC,W ;test
     xorlw b'10000000' ;test        ; toggle PORTC,7 (DIP1)
     movwf PORTC ;test
     return

toggle_test_5  ; test routine
     movf PORTC,W ;test
     xorlw b'00100000' ;test        ; toggle PORTC,5 (DIP2)
     movwf PORTC ;test
     return
#endif ; test_use_DIP_as OUT
#endif ; ms88

#ifdef as88
toggle_test_1  ; test routine
     movf PORTB,W ;test
     xorlw b'10000000' ;test        ; toggle PORTB,7 (LED)
     movwf PORTB ;test
     return

toggle_test_2  ; test routine
     movf PORTA,W ;test
     xorlw b'00000001' ;test        ; toggle PORTA,0 (P_DATA)
     movwf PORTA ;test
     return

toggle_test_3  ; test routine
     movf PORTA,W ;test
     xorlw b'00000010' ;test        ; toggle PORTA,1 (P_CLK)
     movwf PORTA ;test
     return

#ifdef test_use_DIP_as_OUT
toggle_test_4  ; test routine
     movf PORTC,W ;test
     xorlw b'10000000' ;test        ; toggle PORTC,7 (DIP1)
     movwf PORTC ;test
     return

toggle_test_5  ; test routine
     movf PORTC,W ;test
     xorlw b'00100000' ;test        ; toggle PORTC,5 (DIP2)
     movwf PORTC ;test
     return
#endif ; test_use_DIP_as OUT
#endif ; as88

#ifdef as88_2v6
toggle_test_1  ; test routine
     movf PORTB,W ;test
     xorlw b'10000000' ;test        ; toggle PORTB,7 (LED)
     movwf PORTB ;test
     return

toggle_test_2  ; test routine
     movf PORTA,W ;test
     xorlw b'00000001' ;test        ; toggle PORTA,0 (P_DATA)
     movwf PORTA ;test
     return

toggle_test_3  ; test routine
     movf PORTA,W ;test
     xorlw b'00000010' ;test        ; toggle PORTA,1 (P_CLK)
     movwf PORTA ;test
     return

#ifdef test_use_DIP_as_OUT
toggle_test_4  ; test routine
     movf PORTC,W ;test
     xorlw b'10000000' ;test        ; toggle PORTC,7 (DIP1)
     movwf PORTC ;test
     return

toggle_test_5  ; test routine
     movf PORTC,W ;test
     xorlw b'00100000' ;test        ; toggle PORTC,5 (DIP2)
     movwf PORTC ;test
     return
#endif ; test_use_DIP_as OUT
#endif ; as88_2v6
; </editor-fold>
        END

